/***************************************************************************
                             diskpartitioner.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "diskpartitioner.h"

#include <qlayout.h>
#include <qpushbutton.h>

/*A Disk Partitioner provides a user interface around a partition
  view. It allows the user to partition the disk with a bunch of push
  buttons. Furthermore it provides some general information about the
  disk in an information label. */
DiskPartitioner::DiskPartitioner( QWidget *parent, const char *name )
    : QWidget( parent, name)
{
    QVBoxLayout* topVbox = new QVBoxLayout( this, 6);

    QHBoxLayout* hbox = new QHBoxLayout( 6 );
    topVbox->addLayout( hbox );

    view = new PartitionView(this);

    hbox->addWidget( view, 10 );

    QVBoxLayout* vbox = new QVBoxLayout( 6 );
    hbox->addLayout( vbox );

    QPushButton* button;

    button = new QPushButton( this );
    button->setText( tr("&Edit") );
    connect(view, SIGNAL( currentAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    connect( button, SIGNAL( clicked() ), view, SLOT( editCurrent() ) );
    button->setEnabled( FALSE );
    vbox->addWidget( button );

    button = new QPushButton( this );
    button->setText( tr("&Delete") );
    connect(view, SIGNAL( currentAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    connect( button, SIGNAL( clicked() ), view, SLOT( removeOrClearCurrent() ) );
    button->setEnabled( FALSE );
    vbox->addWidget( button );

    vbox->addSpacing( 6 );

    button = new QPushButton( this );
    button->setText( tr("&Add logical") );
    connect( button, SIGNAL( clicked() ), view, SLOT( addLogicalPartition() ) );
    connect(view, SIGNAL( diskAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
    vbox->addWidget( button );


    vbox->addStretch( 10 );

    button = new QPushButton( this );
    button->setText( tr("&Write") );
    connect(view, SIGNAL( changed(bool) ), button, SLOT( setEnabled(bool) ) );
    connect(view, SIGNAL( changed(bool) ), this, SLOT( writeEnabled(bool) ) );
    connect( button, SIGNAL( clicked() ), view, SLOT( writePartitionTable() ) );
    connect( button, SIGNAL( clicked() ), this, SIGNAL( changePartitionTable() ) );
    button->setEnabled( FALSE );
    vbox->addWidget( button );

    button = new QPushButton( this );
    button->setText( tr("&Reset") );
    connect(view, SIGNAL( notOriginal(bool) ), button, SLOT( setEnabled(bool) ) );
    connect( button, SIGNAL( clicked() ), view, SLOT( defaultPartitionTable() ) );
    button->setEnabled( FALSE );
    vbox->addWidget( button );

    view->setFocus();
}


DiskPartitioner::~DiskPartitioner()
{

}


PartitionView* DiskPartitioner::getView()
{
    return view;
}


void DiskPartitioner::writeEnabled(bool b)
{
    emit nextEnabled( this, !b );
}


void DiskPartitioner::showEvent( QShowEvent * )
{
    view->checkState();
}
