/***************************************************************************
                               installselect.h
                             -------------------
    begin                : Sat Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INSTALLSELECT_H
#define INSTALLSELECT_H

class LInlineHelp;

#include "config.h"

#include <alloca.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <rpmlib.h>

#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qwidget.h>
#include <qsocketnotifier.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qlistview.h>

#ifdef GBLI_DEVEL
	#define RPM_PATH "/data/projects/T-1/packages"
#else
	#define RPM_PATH "/mnt/install/packages"
#endif

#define INST_CANCEL -1
#define INST_OK     0
#define INST_ERROR  1
#define INST_NOP    2


class PackageListItem;

class packageInfo {
public:
    Header h;
    char * name, * group;
    bool selected;
//    bool manuallySelected;
    bool inmenu;
    int size;
    void * data;	/* this points to different things for different */
									/* install methods, e.g. filename */
		PackageListItem *item; // the item in the custom selection tree
} ;

class pkgSet {
public:
    packageInfo ** packages;
    int numPackages;
};

class component {
public:
    char * name;
    bool selected;
    bool inmenu;
    pkgSet ps;
};

class componentSet {
public:
    component * comps;  // full components array
    component * base;   // pointer to component with name "Base"
    component * typical;// pointer to component with name "Typical"
    component * minimum;// pointer to component with name "Minimum"
    int numComponents;
};

class callbackInfo {
public:
    packageInfo * pkg;
    int upgrade;
    FD_t fd;
    char *lastFile;
    int totalnum, totalsize;
};

//static int psGetArchScore( const Header h );
//static int pkgCompare( const void * first, const void * second );
//static int pkgCompareVer( const void * first, const void * second );
//static void emptyErrorCallback(void){};

class InstallSelect: public QWidget
{
    Q_OBJECT
public:
    InstallSelect( QWidget * parent, const char * name = 0 );
    ~InstallSelect() {};

		enum { full = 0, typical, base, custom, undefined, upgrade };
		
		int getInstallType() const { return selected; };

		::pkgSet *getPackages(){	return &ps;	};
		bool withX;
		int verifyDependencies();

		int loadData();
		componentSet cs;
		pkgSet ps;

signals:
    void prevEnabled( QWidget *, bool );
    void nextEnabled( QWidget *, bool );

    void showProgressBar();
    void hideProgressBar();
    void resetProgress();
    void setTotalSteps(int);
    void setProgress(int);
    void setProgressLabel(const QString&);

    void installationFinished();
		void setFormatted();

public slots:
		void runUpgrade();
		void runInstall();

protected:
		void showEvent( QShowEvent * );
    void hideEvent( QHideEvent * );

private slots:
    void setSelected( int );
    void showInstallationProgress( int fd );

private:
		int getPackageSet( QString &a_strFileName );
		int getComponentSet( QString &a_strFileName );
		bool skipPackage( char * name );
		void pkgSort();
		void freeComponentSet();
		int doInstall();
//		int doInstall_hack(); // first try ;-)
		void createDirs();
		void selectPackages();
		bool checkForX();	
		void checkFreeSpace();

    QSocketNotifier* notifier;
    pid_t pid;

		int iStage;
    int selected;

    int currentProgress;
    int maxProgress;

    QButtonGroup * gb;
		QRadioButton * rbAll[4];

//		int somefile;
};

#endif

