/***************************************************************************
                          lnetwork.cpp  -  description
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "lnetwork.h"
#include "configfile.h"
#include "sysconfigfile.h"

#include <qlineedit.h>
#include <qvalidator.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qsocket.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>

#include "linlinehelp.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#define NET_DEBUG
#ifdef NET_DEBUG
	# define D(args...)	printf(##args)
#else
	# define D(args...)	do { } while (0)
#endif

LAddressEdit::LAddressEdit( Mode md, QWidget * parent, const char * name )
    : QFrame( parent, name )
{
    m = md;
    e = FALSE;

    int sw = fontMetrics().width( "." );

    b1 = new QLineEdit( this );
    b1->setValidator( new QIntValidator( 0, md == Address ? 223 : 255, b1 ) );
    b1->setFrame( FALSE );
    b1->setAlignment( AlignRight );
    b1->setMaxLength( 3 );
    b1->setText( "0" );

    b2 = new QLineEdit( this );
    b2->setValidator( new QIntValidator( 0, 255, b2 ) );
    b2->setFrame( FALSE );
    b2->setAlignment( AlignRight );
    b2->setMaxLength( 3 );
    b2->setText( "0" );

    b3 = new QLineEdit( this );
    b3->setValidator( new QIntValidator( 0, 255, b3 ) );
    b3->setFrame( FALSE );
    b3->setAlignment( AlignRight );
    b3->setMaxLength( 3 );
    b3->setText( "0" );

    b4 = new QLineEdit( this );
    b4->setValidator( new QIntValidator( 0, 255, b4 ) );
    b4->setFrame( FALSE );
    b4->setAlignment( AlignRight );
    b4->setMaxLength( 3 );
    b4->setText( "0" );

    QHBoxLayout * l = new QHBoxLayout( this, 2, 0 );
    l->addWidget( b1 );
    l->addSpacing( sw );
    l->addWidget( b2 );
    l->addSpacing( sw );
    l->addWidget( b3 );
    l->addSpacing( sw );
    l->addWidget( b4 );
    l->activate();

    setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
    
    b1->installEventFilter( this );
    b2->installEventFilter( this );
    b3->installEventFilter( this );
    b4->installEventFilter( this );

    setFixedSize(sizeHint());
}


LAddressEdit::~LAddressEdit()
{
}


bool LAddressEdit::eventFilter( QObject * o, QEvent * e )
{
    if ( !e || !o )
	return FALSE;

    if ( e->type() == QEvent::KeyPress && o->inherits( "QLineEdit" ) ) {
	QLineEdit * l = (QLineEdit *)o;
	// hack - delete leading zeroes quite often, to keep the length down
	if ( l->text().length() > 1 && l->cursorPosition() &&
	     !l->hasMarkedText() && l->text()[0] == '0' ) {
	    int i = l->cursorPosition();
	    l->setText( l->text().mid( 1, 999 ) );
	    l->setCursorPosition( i-1 );
	}
    }

    if ( e->type() == QEvent::FocusIn && o->inherits( "QLineEdit" ) ) {
	QLineEdit * l = (QLineEdit *)o;
	l->selectAll();
    }

    if ( e->type() == QEvent::FocusOut && o->inherits( "QLineEdit" ) ) {
	QLineEdit * l = (QLineEdit *)o;
	const QValidator * v = l->validator();
	int i = 0;
	QString tmp = l->text();
	if ( tmp.isEmpty() )
	    l->setText( "0" );
	else if ( v && v->validate( tmp, i ) != QValidator::Acceptable )
	    l->setText( (tmp == l->text()) ? tmp : QString("0") ); // anything
	i = l->text().toInt();
	if ( m == Netmask ) {
	    // Turn this into a valid netmask by smearing bits to the left.
	    i |= (i << 1);
	    i |= (i << 2);
	    i |= (i << 4);
	    i &= 255;

	    if ( i > 0 ) {
		if ( o == b2 ) {
		    b1->setText( "255" );
		} else if ( o == b3 ) {
		    b1->setText( "255" );
		    b2->setText( "255" );
		} else if ( o == b4 ) {
		    b1->setText( "255" );
		    b2->setText( "255" );
		    b3->setText( "255" );
		}
	    }
	    if ( i < 255 ) {
		if ( o == b1 ) {
		    b2->setText( "0" );
		    b3->setText( "0" );
		    b4->setText( "0" );
		} else if ( o == b2 ) {
		    b3->setText( "0" );
		    b4->setText( "0" );
		} else if ( o == b3 ) {
		    b4->setText( "0" );
		}
	    }
	}
	if ( i != l->text().toInt() ) {
	    tmp.sprintf( "%u", i );
	    l->setText( tmp );
	}
	return FALSE;
    }

    if ( !e || e->type() != QEvent::KeyPress ||
	 !( ((QKeyEvent*)e)->text() == "." ||
	    ((QKeyEvent*)e)->key() == Key_Enter ||
	    ((QKeyEvent*)e)->key() == Key_Return ||
	    ((QKeyEvent*)e)->key() == Key_Right ||
	    ((QKeyEvent*)e)->key() == Key_Left ||
	    ((QKeyEvent*)e)->key() == Key_Tab ||
	    ((QKeyEvent*)e)->key() == Key_BackTab ) )
	return FALSE;

    if ( ((QKeyEvent*)e)->key() == Key_BackTab ||
	 ((QKeyEvent*)e)->key() == Key_Left ) {

	if ( ((QKeyEvent*)e)->key() == Key_Left &&
	     ((QLineEdit*)o)->cursorPosition() > 0 )
	    return FALSE;

	if ( o == b2 ) {
	    b1->setFocus();
	    b1->setCursorPosition( b1->text().length() );
	} else if ( o == b3 ) {
	    b2->setFocus();
	    b2->setCursorPosition( b2->text().length() );
	} else if ( o == b4 ) {
	    b3->setFocus();
	    b3->setCursorPosition( b3->text().length() );
	} else {
	    return FALSE;
	}

	return TRUE;
    }

    // at this point, we know we're moving right.

    if ( ((QKeyEvent*)e)->key() == Key_Right &&
	 ((QLineEdit*)o)->cursorPosition() < (int)((QLineEdit*)o)->text().length() )
	return FALSE;

    // at this point we know we're REALLY moving right.

    if ( o == b1 ) {
	b2->setFocus();
	b2->setCursorPosition( 0 );
    } else if ( o == b2 ) {
	b3->setFocus();
	b3->setCursorPosition( 0 );
    } else if ( o == b3 ) {
	b4->setFocus();
	b4->setCursorPosition( 0 );
    } else {
	if ( ((QKeyEvent*)e)->key() == Key_Right )
	    return FALSE;
	emit returnPressed(); // it's a lie but...
	return FALSE;
    }
    return TRUE;
}


QString LAddressEdit::text()
{
    return b1->text() + "." + b2->text() + "." + b3->text() + "." + b4->text();
}


struct in_addr
LAddressEdit::address()
{
    struct in_addr	addr;

    if (!inet_aton(text(), &addr))
	addr.s_addr = 0;
    return addr;
}


QSize LAddressEdit::sizeHint() const
{
    return QSize( fontMetrics().width( "255.255.255.255" ) + 20,
		  b1->sizeHint().height() + 4 );
}


QSizePolicy LAddressEdit::sizePolicy() const
{
    return b1->sizePolicy();
}


void LAddressEdit::drawContents( QPainter * p )
{
    p->setPen( NoPen );
    p->fillRect( contentsRect(), colorGroup().base() );
    p->setPen( colorGroup().text() );
    p->drawText( b1->x() + b1->width(), b1->y(),
		 b2->x() - b1->x() - b1->width(), b1->height(),
		 AlignCenter,
		 "." );
    p->drawText( b2->x() + b2->width(), b2->y(),
		 b3->x() - b2->x() - b2->width(), b2->height(),
		 AlignCenter,
		 "." );
    p->drawText( b3->x() + b3->width(), b1->y(),
		 b4->x() - b3->x() - b3->width(), b3->height(),
		 AlignCenter,
		 "." );
}


bool LAddressEdit::edited() const
{
    return b1->edited() || b2->edited() || b3->edited() || b4->edited();
}



void LAddressEdit::setAddress( struct in_addr addr )
{
    unsigned long haddr;
    char	octet[4][10];
    int		i;

    haddr = ntohl(addr.s_addr);
    for (i = 0; i < 4; i++, haddr >>= 8)
	sprintf(octet[i], "%u", (unsigned int) (haddr & 0xFF));

    b1->setText(octet[3]);
    b2->setText(octet[2]);
    b3->setText(octet[1]);
    b4->setText(octet[0]);
}

void LAddressEdit::setAddress( const QString & a )
{
    if ( !a.length() || a == "0.0.0.0" )
	return;

    struct in_addr tmp;
    if ( ::inet_aton( a, &tmp ) == 0 )
	return;
    setAddress(tmp);
}


void LAddressEdit::setEnabled( bool enable )
{
    b1->setEnabled( enable );
    b2->setEnabled( enable );
    b3->setEnabled( enable );
    b4->setEnabled( enable );
    QWidget::setEnabled( enable );
}


LNetwork::LNetwork( LNetinfo *net, QWidget * parent, const char * name )
    : QWidget( parent, name ), info( net )
{

    ipEdit = new LAddressEdit( LAddressEdit::Address, this );
    netmaskEdit = new LAddressEdit( LAddressEdit::Netmask, this );
    routerEdit = new LAddressEdit( LAddressEdit::Router, this );

    ns1Edit = new LAddressEdit( LAddressEdit::Other, this );
    ns2Edit = new LAddressEdit( LAddressEdit::Other, this );
    ns3Edit = new LAddressEdit( LAddressEdit::Other, this );

    nameEdit = new QLineEdit( this );
    nisEdit = new QLineEdit( this );

    proxyEdit = new QLineEdit( this );

    proxyPortEdit = new QLineEdit( this );
    proxyPortEdit->setValidator( new QIntValidator( 0, 32768, proxyPortEdit ) );
    proxyPortEdit->setAlignment( AlignRight );
    proxyPortEdit->setMaxLength( 5 );
    proxyPortEdit->setFixedWidth( fontMetrics().width( "12345" ) + 15 );
    proxyPortEdit->setText( "3128" );

    noproxyforEdit = new QLineEdit( this );

    // Update widget content with data from BOOTP/netprobe we have so far
    updateWidgets();

		QHBoxLayout *loHorz = new QHBoxLayout( this, 5 );

    QGridLayout * grid = new QGridLayout( 1, 1, 10 );
		loHorz->addStretch( 1 );
		loHorz->addLayout( grid, 8 );
		loHorz->addStretch( 1 );

    grid->addColSpacing( 0, 19 );
    grid->setRowStretch( 6, 1 );
    grid->setRowStretch( 9, 1 );
    grid->setRowStretch( 12, 1 );
    grid->setRowStretch( 13, 1 );

/*    LInlineHelp * inlineHelp = new LInlineHelp( this );
    inlineHelp->setDocument( "network_overview.qml" );
    grid->addMultiCellWidget( inlineHelp, 0, 11, 5, 5 );*/
    
    QFrame * hack = new QFrame( this, 0, 0, TRUE );
    hack->setFrameStyle( QFrame::HLine + QFrame::Sunken );
    grid->addMultiCellWidget( hack, 9, 9, 0, 4 );

    bg = new QButtonGroup( 0 );
    QRadioButton *r, *d;

    r = new QRadioButton( tr( "No ethernet" ), this );
    bg->insert( r, LNetinfo::NoEthernet );
    grid->addMultiCellWidget( r, 0, 0, 0, 4 );
    d = new QRadioButton( tr( "Ethernet configured using DHCP" ), this );
    bg->insert( d, LNetinfo::UseDHCP );
    grid->addMultiCellWidget( d, 1, 1, 0, 4 );
    r = new QRadioButton( tr( "Ethernet configured statically" ), this );
    if (net->hadDHCP())
      d->setChecked(TRUE);
    else
      r->setChecked( TRUE );
    bg->insert( r, LNetinfo::UseStaticConfig );
    grid->addMultiCellWidget( r, 2, 2, 0, 4 );

    connect( bg, SIGNAL(clicked(int)),
	     this, SLOT(setMode(int)) );

    grid->addWidget( new QLabel( tr("IP Address"), this ), 3, 1 );
    grid->addWidget( ipEdit, 3, 2, QLabel::AlignLeft );

    grid->addWidget( new QLabel( tr("Netmask:"), this ), 4, 1 );
    grid->addWidget( netmaskEdit, 4, 2, QLabel::AlignLeft );
    grid->addWidget( new QLabel( tr("Gateway:"), this ), 5, 1 );
    grid->addWidget( routerEdit, 5, 2, QLabel::AlignLeft );

		grid->addRowSpacing( 11, 10 );
    grid->addWidget( new QLabel( tr("Hostname"), this ), 12, 1 );
    grid->addMultiCellWidget( nameEdit, 12, 12, 2, 4 );

    grid->addWidget( new QLabel( tr("Name server"), this ), 3, 3 );
    grid->addWidget( ns1Edit, 3, 4, QLabel::AlignLeft );
    grid->addWidget( new QLabel( tr("Backup #1:"), this ), 4, 3 );
    grid->addWidget( ns2Edit, 4, 4, QLabel::AlignLeft );
    //grid->addWidget( new QLabel( tr("Backup #2:"), this ), 5, 3 );
    //grid->addWidget( ns3Edit, 5, 4 );
    ns3Edit->hide();

    grid->addWidget( new QLabel( tr("NIS domain:"), this ), 5, 3 );
    grid->addWidget( nisEdit, 5, 4, QLabel::AlignLeft );

    grid->addWidget( new QLabel( tr("Proxy server:"), this ), 7, 1 );
    grid->addMultiCellWidget( proxyEdit, 7, 7, 2, 3 );
    grid->addWidget( proxyPortEdit, 7, 4, QGridLayout::AlignLeft );
    grid->addWidget( new QLabel( tr("No proxy for:"), this ), 8, 1 );
    grid->addMultiCellWidget( noproxyforEdit, 8, 8, 2, 4 );

    ipEdit->setFocus();

    addInputWidget( ipEdit );
    addInputWidget( netmaskEdit );
    addInputWidget( routerEdit );
    addInputWidget( ns1Edit );
    addInputWidget( ns2Edit );
    addInputWidget( ns3Edit );
    addInputWidget( nameEdit );
    addInputWidget( nisEdit );
//		addInputWidget( proxyEdit );
//		addInputWidget( proxyPortEdit );

    connect( info, SIGNAL(newConfig()),
	     this, SLOT(updateWidgets()) );

    if (net->hadDHCP())
      setMode(LNetinfo::UseDHCP);
    else
      setMode(LNetinfo::UseStaticConfig);
}

LNetwork::~LNetwork()
{
}

//////////////////////////////////////////////////////////////////
// Do return/focus handling
//////////////////////////////////////////////////////////////////
void
LNetwork::addInputWidget(QWidget *edit)
{
    connect( edit, SIGNAL(returnPressed()),
	     this, SLOT(updateInfo()) );
    edit->installEventFilter(this);
}

bool
LNetwork::eventFilter(QObject *, QEvent *e)
{
    if (e->type() == QEvent::FocusIn || e->type() == QEvent::FocusOut)
    	updateInfo();
    return false;
}

//////////////////////////////////////////////////////////////////
// Debug function
//////////////////////////////////////////////////////////////////
#ifdef NET_DEBUG
#define DUMP_WIDGET(w)	\
	printf("%-15s: \"%s\"%s\n", \
		#w, w->text().ascii(), w->edited()? " (edited)" : "")
#define DUMP_ALL_WIDGETS	\
	do {	\
		DUMP_WIDGET(ipEdit); \
		DUMP_WIDGET(netmaskEdit); \
		DUMP_WIDGET(routerEdit); \
		DUMP_WIDGET(ns1Edit); \
		DUMP_WIDGET(ns2Edit); \
		DUMP_WIDGET(nameEdit); \
		DUMP_WIDGET(nisEdit); \
	} while (0)
#endif

//////////////////////////////////////////////////////////////////
// Update displayed values from probe data
//////////////////////////////////////////////////////////////////
void LNetwork::updateWidgets()
{
//    D("updating widgets%s\n", ipEdit? "" : " (no ipEdit)");
    if (!ipEdit)
			return;
//    DUMP_ALL_WIDGETS;

    if (!ipEdit->edited())
        ipEdit->setAddress(info->getAddress());

    if (!netmaskEdit->edited())
        netmaskEdit->setAddress(info->getNetmask());

    if (!routerEdit->edited())
        routerEdit->setAddress(info->getGateway());

    if (!ns1Edit->edited())
        ns1Edit->setAddress(info->getNameserver());

    if (!nameEdit->edited())
				nameEdit->setText(info->getHostname());

    if (!nisEdit->edited())
				nisEdit->setText(info->getNISDomain());
}

//////////////////////////////////////////////////////////////////
// Update network config from user input
//////////////////////////////////////////////////////////////////
void
LNetwork::updateInfo()
{
    bool	changed = false;

//    D("LNetwork::updateInfo()\n");
//    DUMP_ALL_WIDGETS;

    if (nameEdit->edited())
	    info->setHostname(nameEdit->text(), LNetinfo::User);
    if (nisEdit->edited())
	    info->setNISDomain(nisEdit->text(), LNetinfo::User);
    if (routerEdit->edited())
	    info->setGateway(routerEdit->text(), LNetinfo::User);
    if (ns1Edit->edited())
	    info->setNameserver(ns1Edit->text(), LNetinfo::User);
    if (ns2Edit->edited())
	    info->setNameserver2(ns2Edit->text(), LNetinfo::User);
		if ( proxyEdit->edited() )
	    info->setProxyName(proxyEdit->text(), LNetinfo::User);
		if ( proxyPortEdit->edited() )
	    info->setProxyPort(proxyPortEdit->text(), LNetinfo::User);

    if (netmaskEdit->edited()
    	&& info->setNetmask(netmaskEdit->text(), LNetinfo::User))
				changed = true;
    if (ipEdit->edited() && info->setAddress(ipEdit->text(), LNetinfo::User))
			changed = true;
//    D("updateInfo(%schanged)\n", changed? "" : "un");
    if( changed )
			info->probe( true );

#ifdef NET_DEBUG_DUMP
    info->save("/tmp/netdebug");
#endif
}

void LNetwork::setMode( int nm )
{
    bool e = ( nm == LNetinfo::UseStaticConfig );
    ipEdit->setEnabled( e );
    netmaskEdit->setEnabled( e );
    routerEdit->setEnabled( e );
    ns1Edit->setEnabled( e );
    ns2Edit->setEnabled( e );
    ns3Edit->setEnabled( e );
    nisEdit->setEnabled( e );
		e = true;//!( nm == LNetinfo::NoEthernet );
		proxyEdit->setEnabled( e );
		proxyPortEdit->setEnabled( e );
		noproxyforEdit->setEnabled( e );

    info->setMode( (LNetinfo::Setup)nm );
}

//////////////////////////////////////////////////////////////////
// Netinfo item class
//////////////////////////////////////////////////////////////////
LNetinfo::Item::Item(const char *n, const char *v)
: source(Default), name(n), value(v)
{
}

LNetinfo::Item::~Item()
{
    // NOP
}

bool
LNetinfo::Item::set(const char *v, Source s)
{
    if( s < source )
			return false;
    if( !strcmp(value, v) )
			return false;
#ifdef NET_DEBUG
    printf("%s=\"%s\" ", name.ascii(), v);
    switch (s) {
      case Default:
  			printf("(default)"); break;
      case Wildguess:
  			printf("(guessed)"); break;
      case Probed:
  			printf("(probed)"); break;
      case User:
  			printf("(user)"); break;
    }
    printf("\n");
#endif
    source = s;
    value = v;

		// Lets remove strange spaces, cause they r killing the system ;-)
		int pos = 0;
		while( (pos = value.find( " ", pos )) >= 0 )
			value.remove( pos, 1 );

    return true;
}


//////////////////////////////////////////////////////////////////
// Netinfo class
//////////////////////////////////////////////////////////////////
LNetinfo::LNetinfo(void)
: mode(UseStaticConfig),
  address("Address", "192.168.1.2"),
  netmask("Netmask", "255.255.255.0"),
  gateway("Gateway", "0.0.0.0"),
  nameserver("Nameserver", "0.0.0.0"),
  nameserver2("Nameserver2", "0.0.0.0"),
  hostname("Hostname", "noname.nodomain.nowhere"),
	proxy( "Proxy", "" ),
	proxyport( "ProxyPort", "3128" ),
	noproxyfor( "NoProxyFor", "" ),
  nisdomain("NISDomain", ""),
  dhcp(FALSE)
{
    netprobePath = getenv("NETPROBE");
    if (netprobePath == 0)
      netprobePath = "./extrabin/lnetprobe";

    // if running with dhpc information, use it.
    inputBOOTP();

    probeSock = 0;
    probePid = 0;

    startProbe(Idle, false);
}

LNetinfo::~LNetinfo()
{
    stopProbe();
}

//////////////////////////////////////////////////////////////////
// Parse FOO="bar" lines
//////////////////////////////////////////////////////////////////
static inline char *
parseShellAssignment(char *name)
{
    char	*value, *ep;

    if ((value = strchr(name, '=')) == 0)
      return 0;
    *value++ = '\0';

    // Convert "bar" to bar
    if (*value == '"') {
      if ((ep = strchr(++value, '"')) == 0)
        return 0;
      *ep = '\0';
    } else {
      value = strtok(value, " \t\n\r");
    }

    // Skip empty string
    if (value == 0 || *value == '\0')
      return 0;

    return value;
}

//////////////////////////////////////////////////////////////////
// Start/stop netprobe
//////////////////////////////////////////////////////////////////
void
LNetinfo::probe(bool force)
{
    stopProbe();
    if (netmask.source <= Probed) {
				// Run netmask probe first--the rest depends on it.
        startProbe(Netmask, force);
    } else {
				// Do all the other probes
				startProbe(GBLI, force);
    }
}

void
LNetinfo::startProbe(Probe probe, bool force)
{
// Maddy 1999 12 03 - No more probing
// while we don't have netwotk modules loaded
return;

#ifdef GBLI_DEVEL
	return;
#endif
    const char	*argv[16];
    int		i, fd[2], argc;

#if 0
    if (dhcp) {
			D("Skipping requested probe %d; already have BOOTP data\n", probe);
			return;
    }
#endif

    if (probePid) {
      D("Probe %d already running, %s\n",	probe, force? "terminating" : "skip");
			if (!force)
	  	  return;
      stopProbe();
    }

    // Don't start advanced probes unless we have a valid address.
    // Also skip probing for invalid IP addrs.
    if (probe != Idle) {
        struct in_addr	addr;
        unsigned long	haddr;

        if (address.source < User)
	  		  return;
        if (!inet_aton(address.value, &addr))
			    return;
        haddr = ntohl(addr.s_addr);
    	  if (haddr == 0 || (haddr >> 24) == 127 || (haddr >> 24) > 223)
	    		return;
    }

    D("starting probe %d...\n", probe);
    QApplication::flushX();
    ::pipe(fd);

    probePid = ::fork();
    if (probePid < 0) {
			probePid = 0;
			::close(fd[0]);
			::close(fd[1]);
			return;
    }
    if (probePid == 0) {
    	::close(fd[0]);

    	::dup2(::open("/dev/null", O_RDWR), 0);
    	::dup2(fd[1], 1);
    	::dup2(0, 2);

    	i = getdtablesize();
    	while (--i > 2)
    	    ::close( i );

    	argc = 0;
    	argv[argc++] = "netprobe";

    	argv[argc++] = "--flush";
    	argv[argc++] = "--ignore-ifconfig";
    	argv[argc++] = "--nicely";
    	if (probe == Idle) {
    	    argv[argc++] = "--idle";
    	} else {
    	    argv[argc++] = "--address";
    	    argv[argc++] = address.value;
    	    if (probe != Netmask) {
    	        argv[argc++] = "--netmask";
    	        argv[argc++] = netmask.value;
    	    }
    	    argv[argc++] = "--probe";
    	    switch (probe) {
    	    case Netmask:
    		argv[argc++] = "netmask";
    		break;
    	    case GBLI:
    		argv[argc++] = "gbli";
    		break;
    	    default:
    		exit(1);
    	    }
    	}
    	argv[argc] = 0;
    #ifdef NET_DEBUG
    	printf("Invoking");
    	for (int z = 0; argv[z]; z++)
    		printf(" %s", argv[z]);
    	printf("\n");
    #endif
			::nice( 5 );
    	execv(netprobePath, (char **) argv);
    	D("OOPS: %m");
    	exit(1);
    }
printf( "startProbe pid: %d\n", probePid );
    ::close(fd[1]);

    probeSock = new QSocket(fd[0], this);
    probeSock->setMode(QSocket::Ascii);
    connect(probeSock, SIGNAL(readyRead()),
	     this, SLOT(inputNetprobe()) );
    connect(probeSock, SIGNAL(closed()),
	     this, SLOT(inputNetprobe()) );
    currentProbe = probe;
}

void
LNetinfo::stopProbe()
{
    if (probePid != 0) {
			D("Killing current probe %d, pid=%d\n", currentProbe, probePid);
			::kill(probePid, 15);
			::waitpid(probePid, NULL, 0 );
			probePid = 0;
    }
    if (probeSock != 0) {
			delete probeSock;
			probeSock = 0;
    }
}

//////////////////////////////////////////////////////////////////
// Read stuff from the bootpc file
//////////////////////////////////////////////////////////////////
void
LNetinfo::inputBOOTP()
{
    char	buffer[1024], *name, *value;
    int		count;
    FILE	*fp;

    if ((fp = fopen("/tmp/bootpc.out", "r")) == 0)
      return;

    D("Reading BOOTP information.\n");
    dhcp = TRUE;
    while (fgets(buffer, sizeof(buffer), fp) != NULL) {
	count = strlen(buffer);
	if (buffer[count-1] == '\n')
	  buffer[--count] = '\0';

	name = buffer;
	if ((value = parseShellAssignment(name)) == 0)
	  continue;

	if (!strcmp(name, "HOSTNAME")) {
	  hostname.set(value, User);
	} else if (!strcmp(name, "NISDOMAIN")) {
	  nisdomain.set(value, User);
	} else {
	  // Assume it's an address
	  struct in_addr	addr;

	  value[strspn(value, "0123456789.")] = '\0';
	  if (!inet_aton(value, &addr))
	    continue;
	  if (!strcmp(name, "IPADDR"))
	    address.set(value, User);
	  else if (!strcmp(name, "NETMASK"))
	    netmask.set(value, User);
	  else if (!strcmp(name, "GATEWAYS"))
	    gateway.set(value, User);
	  else if (!strcmp(name, "DNSSRVS"))
	    nameserver.set(value, User);
	}
    }
    fclose(fp);
}

//////////////////////////////////////////////////////////////////
// Read stuff from the netprobe pipe
//////////////////////////////////////////////////////////////////
void
LNetinfo::inputNetprobe()
{
    char	buffer[1024], *name, *value;
    bool	update = false;

    if (probeSock == 0)
	return;

    D("Input from netprobe\n");
    while (probeSock->canReadLine() > 0) {
	QString result = probeSock->readLine();

	// Annoying, but parsing with QString is even less readable IMHO.
	D("> %s", (const char *) result);
	if (strlen(result) >= sizeof(buffer))
	  continue;
	strcpy(buffer, result);

	name = buffer;
	if ((value = parseShellAssignment(name)) == 0)
	  continue;

	D("name=%s value=%s\n", name, value);
	if (!strcmp(name, "HOSTNAME")) {
	  hostname.set(value, Probed);
	} else if (!strcmp(name, "NISDOMAIN")) {
	  nisdomain.set(value, Probed);
	} else {
	  // Assume it's an address
	  struct in_addr addr;

	  value[strspn(value, "0123456789.")] = '\0';
	  if (!inet_aton(value, &addr))
	    continue;

	  if (!strcmp(name, "ADDRESS")) {
	    address.set(value, Probed);
	  } else if (!strcmp(name, "NETWORK")) {
	    address.set(value, Wildguess);
	    addr.s_addr = htonl(ntohl(addr.s_addr) + 1);
	    gateway.set(inet_ntoa(addr), Wildguess);
	  } else if (!strcmp(name, "NETMASK")) {
	    netmask.set(value, Probed);
	  } else if (!strcmp(name, "GATEWAY")) {
	    gateway.set(value, Probed);
	  } else if (!strcmp(name, "DNSSERV0")
		  || !strcmp(name, "NAMESERVER")) {
	    nameserver.set(value, Probed);
	  } else {
	    continue;
	  }
	}
	update = true;
    }

    if (update)
        emit newConfig();

    if (probeSock->state() == QSocket::Idle) {
	stopProbe();
	if (currentProbe == Idle)
	  startProbe(Netmask, false);
	if (currentProbe == Netmask)
	  startProbe(GBLI, false);
    }
}

//////////////////////////////////////////////////////////////////
// Set various items from user dialog
//////////////////////////////////////////////////////////////////
bool
LNetinfo::setMode(Setup m)
{
    if (mode == m)
	return false;
    mode = m;
    return true;
}

bool
LNetinfo::setAddress(const char *value, Source src)
{
    return address.set(value, src);
}

bool
LNetinfo::setNetmask(const char *value, Source src)
{
    return netmask.set(value, src);
}

bool
LNetinfo::setGateway(const char *value, Source src)
{
    return gateway.set(value, src);
}

bool
LNetinfo::setNameserver(const char *value, Source src)
{
    return nameserver.set(value, src);
}

bool
LNetinfo::setNameserver2(const char *value, Source src)
{
    return nameserver2.set(value, src);
}

bool
LNetinfo::setHostname(const char *name, Source src)
{
    return hostname.set(name, src);
}

bool
LNetinfo::setNISDomain(const char *name, Source src)
{
    return nisdomain.set(name, src);
}

bool
LNetinfo::setProxyName(const char *name, Source src)
{
    return proxy.set(name, src);
}

bool
LNetinfo::setProxyPort(const char *name, Source src)
{
    return proxyport.set(name, src);
}

void
LNetinfo::save(const char *root)
{
    if( getenv("NET_ROOTDIR") != 0 )
			root = getenv("NET_ROOTDIR");
    else if( root == 0 )
			root = "./root";

    saveEther(root);
    saveResolv(root);
#ifdef NO_USED
    saveNIS(root);
#endif
    saveSysNetwork(root);
		saveProxy(root);
		saveHosts( root );
#ifdef NO_USED
    saveTmpNetwork();
#endif
}

bool
LNetinfo::saveEther(const char *root)
{
    struct in_addr	addr, mask, broadcast, network;

    D("Save ether, mode=%d\n", mode);
    if( mode == NoEthernet )
			return false;

    SysConfigFile config( root, "/etc/sysconfig/network-scripts/ifcfg-eth0" );

    inet_aton(address.value, &addr);
    inet_aton(netmask.value, &mask);
    broadcast.s_addr = addr.s_addr | ~(mask.s_addr);
    network.s_addr = addr.s_addr & mask.s_addr;

/*  config.append("#!/bin/sh");
    config.append("#>>>Device type: ethernet");
    config.append("#>>>Variable declarations:");

    config.set("DEVICE",	"eth0" );
    config.set("IPADDR",	address.value);
    config.set("NETMASK",	netmask.value);
    config.set("NETWORK",	inet_ntoa(network));
    config.set("BROADCAST",	inet_ntoa(broadcast));
    //config.set("GATEWAY",	gateway.value);
    config.set("ONBOOT",	"yes");
    config.set("DYNAMIC",	(mode == UseStaticConfig)? "" : "dhcp");

    config.append("#>>>End variable declarations");*/
    config.set("DEVICE", "eth0", false );
    config.set("BOOTPROTO", (mode == UseStaticConfig) ? "none" : "dhcp", false );
    config.set("IPADDR", address.value, false );
    config.set("NETMASK", netmask.value, false );
    config.set("NETWORK", inet_ntoa(network), false );
    config.set("BROADCAST", inet_ntoa(broadcast), false );
    config.set("ONBOOT", "yes", false );

    return config.save();
}

bool LNetinfo::saveResolv(const char *root)
{

    FILE	*fp;
    int		i;

    ConfigFile config( root, "/etc/resolv.conf" );

    fp = config.open("a");
    if (fp == 0)
			return false;

    if ((i = hostname.value.find('.')) > 0) {
			QString	domain = hostname.value.mid(i+1, (unsigned int) -1);

			fprintf(fp, "domain %s\nsearch %s\n",
			  domain.ascii(), domain.ascii() );
    }

    if( mode == NoEthernet ){
	    fclose(fp);
			return false;
		}

    if (nameserver.value != "0.0.0.0")
			fprintf(fp, "nameserver %s\n", nameserver.value.ascii());
    if (nameserver2.value != "0.0.0.0")
			fprintf(fp, "nameserver %s\n", nameserver2.value.ascii());

    fclose(fp);
    return true;
}

#ifdef NOT_USED
bool
LNetinfo::saveNIS(const char *root)
{
    FILE	*fp;

    if (nisdomain.value.isEmpty())
			return true;

    ConfigFile config( root, "/etc/nis.conf" );
    fp = config.open( "a" );
    if (fp == 0)
			return false;
    fprintf( fp, "domain %s broadcast\n", nisdomain.value.ascii() );
    fclose( fp );
    return true;
}
#endif

bool LNetinfo::saveProxy( const char *root )
{
  if( mode == NoEthernet ) return false;

	if( !proxy.value.length() ) return false;
  //BestLinux TODO: save appropriate proxy settings into some other progs ?!
  FILE *fp;

	int ind = 0;
	QString strProxy( proxy.value );
	while( (ind = strProxy.find( "," )) != -1 )
		strProxy = strProxy.replace( ind, 1, "." );
	while( (ind = strProxy.find( "\\" )) != -1 )
		strProxy = strProxy.replace( ind, 1, "/" );
	char delim[] = " \"\'()[]{}<>";
	for( int i = 0; delim[i]; i++ )
		while( (ind = strProxy.find( delim[i] )) != -1 )
			strProxy = strProxy.remove( ind, 1 );
	if( !proxy.value.length() ) return false;
	if( (ind = strProxy.find( "http://" )) == -1 ){
		strProxy = "http://"+ strProxy;
	}else if( ind > 0 )
		return false;
	if( proxyport.value.length() )
		strProxy += ":"+ proxyport.value;

	ConfigFile config( root, "/etc/profile" );
  fp = config.open( "a" );
  if( fp ){
	  fprintf(fp,"\n# Proxy settings\n\n");
  	fprintf(fp,"export http_proxy=\"%s\"\n", strProxy.ascii() );
	  fprintf(fp,"export https_proxy=\"%s\"\n", strProxy.ascii() );
  	fprintf(fp,"export ftp_proxy=\"%s\"\n", strProxy.ascii() );
	  fclose(fp);
	}

	config = ConfigFile( root, "/usr/share/config/kfmrc" );
  fp = config.open( "a" );
  if( fp ){
    fprintf(fp,"\n[Browser Settings/Proxy]\n");
    fprintf(fp,"HTTP-Proxy=%s\n", strProxy.ascii() );
		if( noproxyfor.value.length() )
	    fprintf(fp,"NoProxyFor=%s\n", noproxyfor.value.ascii() );
    fprintf(fp,"UseProxy=Yes\n" );
    fprintf(fp,"FTP-Proxy=%s\n", strProxy.ascii() );
    fclose(fp);
	}

	config = ConfigFile( root, "/etc/skel/.kde/share/config/kfmrc" );
  fp = config.open( "a" );
  if( fp ){
    fprintf(fp,"\n[Browser Settings/Proxy]\n");
    fprintf(fp,"HTTP-Proxy=%s\n", strProxy.ascii() );
		if( noproxyfor.value.length() )
	    fprintf(fp,"NoProxyFor=%s\n", noproxyfor.value.ascii() );
    fprintf(fp,"UseProxy=Yes\n");
    fprintf(fp,"FTP-Proxy=%s\n", strProxy.ascii() );

    fclose(fp);
	}
	return true;
}

bool
LNetinfo::saveSysNetwork(const char *root)
{
    SysConfigFile config( root, "/etc/sysconfig/network" );
/* Old GBLI code
    config.load();
    config.set("NETWORKING",	"yes");
    config.set("IF_LIST",	"dynamic"); // as per raymund's request
    config.set("HOSTNAME",	hostname.value);
    config.append( "# IF_LIST set to dynamic by GBLI\n" );
*/
    // Red Hat seems like new. :)
    config.load();
    config.set("NETWORKING", (mode == NoEthernet?"No":"yes"), false );
    config.set("FORWARD_IPV4",	"false", false );

		QString host = hostname.value;
    int point = host.find( '.' );
		if( point >= 0 ){
	    config.set("HOSTNAME", host.left( point ), false );
	    config.set("DOMAINNAME", host.right( host.length() - point - 1 ), false );
		}else{
	    config.set("HOSTNAME", host, false );
	    config.set("DOMAINNAME", nisdomain.value, false );
		}

    if( mode != NoEthernet ){
      if ( !strstr( gateway.value, "0.0.0.0" )  ) {
  			config.set("GATEWAY", gateway.value, false );
        config.set("GATEWAYDEV", "eth0", false ); //FIXME: Agree what  it always eth0
      }
      if ( !strstr( nameserver.value, "0.0.0.0") ) {
  			config.set("NS1", nameserver.value, false );
      }
      if( !strstr( nameserver2.value, "0.0.0.0" ) ) {
  			config.set("NS2", nameserver2.value, false );
      }
		}

    return config.save();
}

#ifdef NO_USED
bool
LNetinfo::saveTmpNetwork(void)
{
    SysConfigFile config( "", "/tmp/network" );

    if (mode != NoEthernet)
		config.set("CONF_eth0_STAT",	"up");

    config.set("CONF_eth0_IP",		address.value);
    config.set("CONF_eth0_MASK",	netmask.value);
    config.set("CONF_eth0_DYNAMIC",	(mode == UseDHCP)? "dhcp" : "");
    config.set("CONF_ROUTER1_IP",	gateway.value);
//    config.set("CONF_FQ_HOSTNAME",	hostname.value);
    config.set("CONF_DNS_SERVER1_IP",	nameserver.value);
    config.set("CONF_DNS_SERVER2_IP",	nameserver2.value);
    //config.set("CONF_DNS_SERVER3_IP",	nameserver3.value);
    config.set("CONF_NIS_DOMAIN",	nisdomain.value);
    config.set("CONF_NIS_SERVER1_IP",	"");
    config.set("CONF_NIS_SERVER2_IP",	"");
    config.set("CONF_NIS_SERVER3_IP",	"");
    config.set("CONF_PROXY_NAME",	proxy.value);
    config.set("CONF_PROXY_PORT",	proxyport.value);
    config.set("CONF_NO_PROXY",	noproxyfor.value );

    return config.save();
}
#endif

bool
LNetinfo::saveHosts(const char *root)
{
  FILE *fp;

	ConfigFile config( root, "/etc/hosts" );
  fp = config.open( "a" );
  if( fp ){
		fprintf( fp, "127.0.0.1\tlocalhost\tlocalhost.localdomain\n" );
		if( mode == UseStaticConfig && strcmp( address.value, "192.168.1.2" ) && strcmp( hostname.value, "noname.nodomain.nowhere" ) ){
			int i = 0;
			char name[128];
			memset( name, 0, sizeof( name ) );
			strcpy( name, hostname.value.ascii() );
			while( name[i] && name[i] != '.' ){
				i++;
			}
			name[i] = 0;
			fprintf( fp, "%s        %s       %s\n", address.value.ascii(), name, hostname.value.ascii() );
		}
   	fclose( fp );
		return true;
  }
	return false;
}



