/***************************************************************************
                               ltimezone.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "config.h"
#include "ltimezone.h"
#include "linlinehelp.h"
#include "liconloader.h"

#include <qlistview.h>
#include <qlayout.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qlist.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qtl.h>
#include <qvaluelist.h>

#include <unistd.h>


LZoneInfo::LZoneInfo( const QString& line, bool coords )
{
    
    coordinates = coords;
    
    if ( coordinates ) { //zone.tab format
	country = line.left(2);
	int i = 3;
	while (i < int(line.length() ) && line[i].isSpace())
	    i++;
	QString tmpnum = line.mid(i, 3);
	tmpnum += '.';
	i += 3;
	while (i < int(line.length() ) && char(line[i]) >= '0' && char(line[i]) <= '9' ){
	    tmpnum += line[i];
	    i++;
	}
	y = tmpnum.toDouble();

	tmpnum = line.mid(i, 4);
	tmpnum += '.';
	i += 4;
	while (i < int(line.length() ) && char(line[i]) >= '0' && char(line[i]) <= '9' ){
	    tmpnum += line[i];
	    i++;
	}
	x = tmpnum.toDouble();

	while (i < int(line.length() ) && line[i].isSpace())
	    i++;

	while (i < int(line.length() ) && !line[i].isSpace()) {
	    city += line[i];
	    i++;
	}
	while (i < int(line.length() ) && line[i].isSpace())
	    i++;
	while (i < int(line.length() ) ){
	    comment += line[i];
	    i++;
	}
    }
    else { // extrazone.tab format
	int i = 0;
	while (i < int(line.length() ) && !line[i].isSpace()) {
	    city += line[i];
	    i++;
	}
	while (i < int(line.length() ) && line[i].isSpace())
	    i++;
	while (i < int(line.length() ) ){
	    comment += line[i];
	    i++;
	}
    }
}

#include "lxserver.h"

LWorld::LWorld( QWidget * parent, const char * name )
    : QWidget( parent, name)
{
    setWFlags( WResizeNoErase );

    zones.setAutoDelete( TRUE );
    w = h = relw = relh = ox = oy = 0;
    setMouseTracking( TRUE );

    high = 0;

		if( LXServer::runSVGA )
	    worldpm.load( "graphics/world.gif" );
		else
	    if( !worldpm.load( "graphics/world16.gif" ) )
		    worldpm.load( "graphics/world.gif" );

    w = worldpm.width();
    h = worldpm.height();
    setFixedSize( worldpm.size() );

    relw = w - 0;
    relh = h + 12; //24 for the big one

    ox = w/2 + 6; //12 for the big one
    oy = h/2 + 6; //12 for the big one

    cityIndicator
	= new QLabel(0, 0, WStyle_Customize | WStyle_NoBorder | WStyle_Tool );
    cityIndicator->setAutoResize( TRUE );
    cityIndicator->setFrameStyle( QFrame::Box | QFrame::Plain);
    cityIndicator->setBackgroundMode( PaletteBase );

}


LWorld::~LWorld()
{
    delete cityIndicator;
    cityIndicator = 0;
}



void LWorld::readInZoneInfo( QComboBox * combo )
{
    if ( !combo || combo->count() )
	return;

    // extra timezones
    {
	QFile f ("extrazone.tab"); 
	if ( f.open( IO_ReadOnly) ) {
	    QTextStream t(&f);
	    while ( !t.atEnd() ) {
		QString line = t.readLine();
		if (line[0] != '#') {
		    zones.append( new LZoneInfo( line, FALSE ) );
		    zoneDict.insert( zones.last()->city, zones.last() );
		    combo->insertItem( zones.last()->city );
		}
	    }
	    f.close();
	}
    }

    QStringList l;
    {
	QFile f ("zone.tab"); 
	if ( f.open( IO_ReadOnly) ) {
	    QTextStream t(&f);
	    while ( !t.atEnd() ) {
		QString line = t.readLine();
		if (line[0] != '#') {
		    zones.append( new LZoneInfo( line ) );
		    l.append( zones.last()->city );
		    zoneDict.insert(zones.last()->city, zones.last() );
		}
	    }
	    f.close();
	}
    }

    qBubbleSort( l.begin(), l.end() );
    for ( QStringList::Iterator i = l.begin(); i != l.end(); ++i )
	combo->insertItem( *i );
}


LZoneInfo* LWorld::nearestZone( int x, int y)
{
    double d = -1;
    LZoneInfo* result = 0;
    for (LZoneInfo* i = zones.first(); i; i = zones.next() ) {
	if (!i->coordinates)
	    continue;
	int cy = (int) (oy - relh * i->y/180);
	int cx = (int) (ox + relw * i->x/360);
	if (cx > w )
	    cx = cx-w;
	double nd = (cx-x)* (cx-x)+ (cy-y)* (cy-y);
	if ( (d < 0 || nd < d) && !(cx >= x && cy >= y) ) {
	    d = nd;
	    result = i;
	}
    }
    return result;
}

QString LWorld::getComment( const QString& zone)
{
    LZoneInfo* i = zoneDict[zone];
    if (i)
	return  i->comment;
    return QString::null;
}


void LWorld::highlight( const QString& arg)
{
    LZoneInfo* oldHigh = high;
    if (!high || high->city != arg)
	high = zoneDict[arg];

    if (!high)
	return;

    if (high != oldHigh && oldHigh) {
	int y = (int) (oy - relh * oldHigh->y/180);
	int x = (int) (ox + relw * oldHigh->x/360);
	if (x > w )
	    x = x-w;
	repaint( x-6, y-6, 13, 13, FALSE );
    }

    if (!high->coordinates) {
	high = 0;
	cityIndicator->setText("");
	cityIndicator->hide();
	return;
    }

    int y = (int) (oy - relh * high->y/180);
    int x = (int) (ox + relw * high->x/360);
    if (x > w )
	x = x-w;
    repaint( x-6, y-6, 13, 13, FALSE );
    QString s = high->city;
    if ( s.findRev('/') != -1 )
	s = s.right( s.length() - s.findRev('/') - 1 );
    if ( s != cityIndicator->text() )
	cityIndicator->setText( s );
    if ( !cityIndicator->isVisible() )
	cityIndicator->hide();
    emit highlighted(high->city);
}


void LWorld::mouseMoveEvent( QMouseEvent* e)
{
    cityIndicator->move( e->globalPos() + QPoint( 10, 10) );
    LZoneInfo* i = nearestZone( e->pos().x(), e->pos().y() );
    if (i && i != high ) {
	if (high) {
	    int y = (int) (oy - relh * high->y/180);
	    int x = (int) (ox + relw * high->x/360);
	    if (x > w )
		x = x-w;
	    high = 0;
	    repaint( x-6, y-6, 13, 13, FALSE );
	}
	high = i;
	highlight( high->city );
	QApplication::syncX(); // for VGA16
    }
}


void LWorld::enterEvent( QEvent * )
{
    if ( !cityIndicator->text().isEmpty() )
	cityIndicator->show();
    cityIndicator->raise();
}


void LWorld::leaveEvent( QEvent * )
{
    cityIndicator->hide();
}


void LWorld::resizeEvent( QResizeEvent* )
{
}


void LWorld::paintEvent( QPaintEvent*  e)
{
    QPainter p( this );
    p.setClipRegion( e->region() );
    p.setClipping( TRUE );

    p.drawPixmap( 0,0, worldpm );

    p.setPen( black );

    for (LZoneInfo* i = zones.first(); i; i = zones.next() ) {
	int y = (int) (oy - relh * i->y/180);
	int x = (int) (ox + relw * i->x/360);
	if (x > w )
	    x = x-w;
	p.drawEllipse( x-1, y-1, 3, 3);
    }

    if ( high ) {
	int y = (int) (oy - relh * high->y/180);
	int x = (int) (ox + relw * high->x/360);
	if (x > w )
	    x = x-w;
	p.setPen( white );
	p.setBrush( white );
	p.drawEllipse( x-1, y-1, 3, 3);
	p.setBrush( NoBrush );
	p.drawEllipse( x-6, y-6, 13, 13);
	p.setPen( black );
    }
	

}


void LWorld::mouseReleaseEvent( QMouseEvent * )
{
    if (high)
	emit selected(high->city);
}


LTimeZone::~LTimeZone()
{
}


LTimeZone::LTimeZone( const QString& language, QWidget * parent,
		      const char * name )
    : QWidget( parent, name)
{

    lang = language;
		QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 1 );
    lh->addLayout( l, 8 );
		lh->addStretch( 1 );

    world = new LWorld( this );

    l->addWidget( world );

    comboZone = new QComboBox( this );
    l->addWidget( comboZone );
    
    comment = new QLabel( "|", this );
    comment->setAlignment( AlignTop );
    QFont commentFont = comment->font();
    commentFont.setItalic( TRUE );
    comment->setFont( commentFont );
    l->addWidget( comment, 10 );
    //l->addStretch( 2 );

    connect( world, SIGNAL( selected(const QString&) ),
	     this, SLOT( setTimezone(const QString&) ) );

    connect( comboZone, SIGNAL( activated(const QString&) ),
	     world, SLOT( highlight(const QString&) ) );
    
    connect( comboZone, SIGNAL( activated(const QString&) ),
	     this, SLOT( setTimezone(const QString&) ) );


    radioLocaltime
			= new QRadioButton( tr("Hardware clock runs in &local time"), this );
    radioLocaltime->setChecked( TRUE );
    radioGMT
			= new QRadioButton( tr("Hardware clock runs in &GMT time"), this );

    QButtonGroup* g = new QButtonGroup( this );
    g->hide();
    g->insert( radioGMT );
    g->insert( radioLocaltime );

		l->addStretch( 1 );
    l->addWidget( radioLocaltime );
		l->addSpacing( 3 );
    l->addWidget( radioGMT );
		l->addStretch( 2 );

/*    LInlineHelp* inlineHelp = new LInlineHelp( this );
    inlineHelp->setDocument( "timezone_overview.qml" );
    lh->addWidget( inlineHelp );*/

    prepared = FALSE;
}


void LTimeZone::showEvent( QShowEvent* )
{
    prepare();
}


void LTimeZone::prepare()
{
    if ( prepared )
			return;
    prepared = TRUE;

    world->readInZoneInfo( comboZone );

    // set default timezones for each language
    if ( lang == "german" )
			setTimezone("Europe/Berlin");
    else if ( lang == "italian" )
			setTimezone("Europe/Rome");
    else if ( lang == "french" )
			setTimezone("Europe/Paris");
    else if ( lang == "estonian" )
			setTimezone("Europe/Tallinn");
    else if ( lang == "finnish" )
			setTimezone("Europe/Helsinki");
    else if ( lang == "swedish" )
			setTimezone("Europe/Stockholm");
    else if ( lang == "norwegian" )
			setTimezone("Europe/Oslo");
    else if ( lang == "french" )
			setTimezone("Europe/Paris");
    else if ( lang == "ukrainian" )
			setTimezone("Europe/Kiev");
    else if ( lang == "spanish" )
			setTimezone("Europe/Madrid");
    else if ( lang == "icelandic" )
			setTimezone("Atlantic/Reykjavik");
    else if ( lang == "russian" )
			setTimezone("Europe/Moscow");
    else
			setTimezone("Europe/London");

    world->highlight( timezone() );
}



void LTimeZone::setTimezone( const QString& z)
{
    if (comboZone->currentText() != z ) {
    	for (int i = 0; i < comboZone->count(); i++) {
        if (z == comboZone->text(i) ) {
		   		comboZone->setCurrentItem( i );
        }
    	}
    }
    
    comment->setText( world->getComment( z ) );
}

QString LTimeZone::timezone()
{
    return comboZone->currentText();
}

void LTimeZone::finish( const char * directory )
{
  // write the entry in /etc/sysconfig/clock
  QDir d;
  d.mkdir( QString( directory ) + "/sysconfig" );
  QFile cf( QString( directory ) + "/sysconfig/clock" );
  if( cf.open( IO_WriteOnly ) ){
    QTextStream clock(&cf);
		clock << "UTC=" << (radioGMT->isChecked() ? "true" : "false") << endl;
		clock << "ARC=false" << endl; // ???
		clock << "ZONE=\"" << timezone() << "\"" << endl;
    cf.close();
  }

  // create the /etc/localtime symlink
	char buf[128];
	sprintf( buf, "%s/localtime", directory );
  ::unlink( buf );
  ::symlink( ("/usr/share/zoneinfo/"+ timezone()).ascii(), buf );
}


