/***************************************************************************
                               lwizard.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "lwizard.h"
#include "config.h"
#include "swapper.h"
#include "partitionview.h"
#include "mountpoint.h"

LWizard::LWizard( const QString& a_lang )
    : QWizard( 0, 0, FALSE ), lang( a_lang ), translator(NULL), upgrade(false)
{
    setupTranslator( lang );
    setGeometry( 0, 0, 640, 480 );
    setFixedSize( 640, 480 );
		setFocusPolicy( QWidget::StrongFocus );

    cancelButton()->hide();
    init();
}

LWizard::~LWizard()
{
}

void LWizard::setupTranslator( const QString& lang_ )
{
    lang = lang_;
    languagePath = QDir::currentDirPath() +"/"+ lang +"/";

		QFont::locale_init();

/*		QFont font = QApplication::font();
//    QFont font( "helvetica" );
//    font.setPointSize( 12 );
		if( lang == "russian" ){
			font.setCharSet( QFont::KOI8R );
		}else{
			font.setCharSet( QFont::Latin1 );
		}
		QFont::setDefaultFont( font );*/
//		qApp->setFont( font );

    // set up the language
		if( translator ){
			qApp->removeTranslator( translator );
			delete translator;
		}
		translator = new QTranslator( qApp );
    if( translator->load( "gbli.qm", languagePath ) )
			qApp->installTranslator( translator );

    helpButton()->setText( tr( "Help" ) );
helpButton()->hide();
    nextButton()->setText( tr( "Next>>" ) );
    finishButton()->setText( tr( "Finish" ) );
    backButton()->setText( tr( "<<Back" ) );

		QApplication::syncX();
		QApplication::flushX();
}

void LWizard::init()
{
    videoCard = 0;
    monitor = 0;
    videoMode = 0;
    target = 0;
    fdisk = 0;
    disksel = 0;
    partitionselRoot = 0;
    partitionInfo = 0;
		installselect = 0;
		packageselect = 0;
    mouse = 0;
    network = 0;
    keyboard = 0;
    timezone = 0;
    createusers = 0;
    grub = 0;
    sound = 0;
		finalpage = 0;
		upgradepage = 0;

#ifndef GBLI_DEVEL
    static Atom noDecoration = ::XInternAtom( qt_xdisplay(),
					       "KWM_WIN_DECORATION", False );
		long l = 0;// 0 == KWM::noDecorationx
    XChangeProperty( qt_xdisplay(), winId(), noDecoration, noDecoration, 32,
		  PropModeReplace, (unsigned char *)&l, 1 );
#endif

//    helpBrowser = 0;

    versionLabel = new QLabel(tr("Version: ")+GBLI_VERSION, this);
    versionLabel->show();

    progressLabel = new QLabel( tr("Packages"), this );	
    progressLabel->hide();
    progressBar = new QProgressBar( this );
    progressBar->setMinimumWidth(170);
    progressBar->setMaximumWidth(250);
    progressBar->hide();

    titleLogo = new QLabel( this );
    titleLogo->setPixmap( LIconLoader::loadIcon("bestlogo.xpm"));

    titleLabel = new QLabel( this );
		QFont font( QFont::defaultFont() );
		font.setPointSize( 24 );
    titleLabel->setFont( font );

    QAccel * a = new QAccel( this );
    a->connectItem( a->insertItem( Qt::CTRL + Qt::ALT + Qt::Key_Delete ),
		    this, SLOT(reboot()) );
    a->connectItem( a->insertItem( Qt::CTRL + Qt::ALT + Qt::Key_D ),
		    this, SLOT(screendump()) );
    a->connectItem( a->insertItem( Qt::CTRL + Qt::ALT + Qt::Key_V ),
		    this, SLOT(version()) );
    a->connectItem( a->insertItem( Qt::ALT + Qt::Key_X ),
		    this, SLOT(xterm()) );

#ifdef GBLI_DEVEL
    a->connectItem( a->insertItem( Qt::CTRL + Qt::ALT + Qt::Key_N ),
		    this, SLOT(next()) );

	    //Pages testing zone	
/*  	InstallSelect *install2 = new InstallSelect( this );
    PackageSelect *packageselect1 = new PackageSelect( this, install2 );
    addPage( packageselect1, tr( "Select Packages" ) );*/

//    monitor = new LMonitor( this );
//    addPage( monitor, tr( "Select Monitor" ) );

//		finalpage = new FinalPage( this, lang );
//		addPage( finalpage, tr( "While Best Linux is being installed..." ) );
      //End pages testing zone
//return;
#endif

    	// Mouse setup
      mouse = new LMouse( this );
      addPage( mouse, tr( "Configure mouse" ) );

/*		FinalPage *finalpage1 = new FinalPage( this, lang );
		addPage( finalpage1, tr( "While Best Linux is being installed..." ) );
    setFinishEnabled( finalpage1, TRUE );
return;*/
/*			upgradepage = new UpgradePage( this );
			addPage( upgradepage, tr( "Installation Path" ) );*/

      // partition disks
      fdisk = new DiskPartitioner( this );
      connect( fdisk, SIGNAL( nextEnabled(QWidget*, bool) ),
         this, SLOT( setNextEnabled(QWidget*, bool) ) );

      // target
      target = new Target( fdisk->getView(), this );
      addPage( target, tr( "Select installation path" ) );

      // fdisk is _behind_ target, naturally
      addPage( fdisk, tr( "Partition Disks" ) );

      // disk selector
      disksel = new DiskSelector( fdisk->getView(), this );
      connect( disksel, SIGNAL( nextEnabled(QWidget*, bool) ),
         this, SLOT( setNextEnabled(QWidget*, bool) ) );
      connect( fdisk, SIGNAL( changePartitionTable() ),
         disksel, SLOT( partitionTableChanged() ) );
      addPage( disksel, tr("Select Hard Disk") );

      // select root partition
      partitionselRoot = new PartitionSelector( fdisk->getView(), PartitionSelector::OnlyRootMode, this );
      addPage( partitionselRoot, tr( "Root Partition" ) );

      // partition info
      partitionInfo = new PartitionInfo( fdisk->getView(), this );
      connect( partitionInfo, SIGNAL( nextEnabled(QWidget*, bool) ),
         this, SLOT( setNextEnabled(QWidget*, bool) ) );
      addPage( partitionInfo, tr( "Partition Information" ) );

	    QTimer::singleShot( 500, ((LWizard*)this), SLOT(setupPages2()) );
}

void LWizard::setupPages2()
{
    // select packages
	  installselect = new InstallSelect( this );
    connect( installselect, SIGNAL( nextEnabled(QWidget*, bool) ),
	     this, SLOT( setNextEnabled(QWidget*, bool) ) );
  	addPage( installselect, tr( "Select Installation" ) );

    connect( installselect, SIGNAL(installationFinished()),
	     this, SLOT(packageInstallationFinished()) );
    connect( installselect, SIGNAL( prevEnabled(QWidget*, bool) ),
	     this, SLOT( setBackEnabled(QWidget*, bool) ) );

    connect( installselect, SIGNAL( showProgressBar() ),
	     progressBar, SLOT( show() ) );
    connect( installselect, SIGNAL( showProgressBar() ),
	     progressLabel, SLOT( show() ) );
    connect( installselect, SIGNAL( hideProgressBar() ),
	     progressBar, SLOT( hide() ) );
    connect( installselect, SIGNAL( hideProgressBar() ),
	     progressLabel, SLOT( hide() ) );

    connect( installselect, SIGNAL( setProgressLabel(const QString&) ),
	     progressLabel, SLOT( setText(const QString&) ) );
    connect( installselect, SIGNAL( resetProgress() ),
	     progressBar, SLOT( reset() ) );
    connect( installselect, SIGNAL( setTotalSteps(int) ),
	     progressBar, SLOT( setTotalSteps(int) ) );
    connect( installselect, SIGNAL( setProgress(int) ),
	     progressBar, SLOT( setProgress(int) ) );

    packageselect = new PackageSelect( this, installselect );
    addPage( packageselect, tr( "Select individual packages" ) );

    // keyboard
    keyboard = new LKeyboard( lang, this );
    addPage( keyboard, tr( "Keyboard Type" ) );

    // select video card and stuff
    videoCard = new LVideoCard( this );
    monitor = new LMonitor( this );
    videoMode = new LVideoModeSelector( monitor, videoCard,
      					mouse, keyboard, this );
    addPage( videoCard, tr( "Select Video Card" ) );
    addPage( monitor, tr( "Select Monitor" ) );
    addPage( videoMode, tr( "Select Video Mode" ) );

    connect( videoCard, SIGNAL(nextAvailable(QWidget *,bool)),
  	   this, SLOT(setNextEnabled(QWidget *,bool)) );

#ifndef GBLI_DEVEL
    // setup sound card
//    sound = new LSound(this);
//    addPage(sound, tr("Test Sound Card"));
#endif

    // setup some users
    createusers = new LCreateUsers( this );
    connect( createusers, SIGNAL( nextEnabled(QWidget*, bool) ),
	     this, SLOT( setNextEnabled(QWidget*, bool) ) );
    addPage( createusers, tr( "Setup Users" ) );

    // set up network
    netdata = new LNetinfo;
    network = new LNetwork( netdata, this );
    addPage( network, tr( "Configure networking" ) );

    // GRUB
    grub = new GRUB( fdisk->getView(), this );
    addPage( grub, tr("Linux Loader") );

    // set up timezone
    timezone = new LTimeZone( lang, this );
    addPage( timezone, tr( "Configure Time Zone" ) );

		finalpage = new FinalPage( this, lang );
		addPage( finalpage, tr( "While Best Linux is being installed..." ) );
	  setHelpEnabled( finalpage, FALSE );
	  setNextEnabled( finalpage, FALSE );
    setFinishEnabled( finalpage,
#ifdef GBLI_DEVEL
			TRUE );			
#else
			FALSE );
#endif
}

bool LWizard::appropriate( QWidget * w ) const
{
	if( w == mouse || w == upgradepage || w == finalpage )
		return true;
	
	if( upgrade )
		return false;

	if( w == keyboard ){
		((LWizard*)this)->setBackEnabled( keyboard, false );// no way back
 	  QTimer::singleShot( 200, installselect, SLOT(runInstall()) );
 	  QTimer::singleShot( 100, timezone, SLOT(prepare()) );
		return true;
	}

	if( w == packageselect ){
		return installselect->getInstallType() == InstallSelect::custom;
	}

  if ( w == fdisk )
		return target->selected() == Target::Custom;

  if ( w == disksel )
		return target->selected() == Target::Disk;

  if ( w == partitionselRoot ) {
  	if ( target->selected() != Target::Partitions ) {
 	    // if the user did not chose a root partition yet, we have
 	    // to ask him
 	    return MountPoints::mountPartition( "/" ).isEmpty();
  	}else{
	    QApplication::setOverrideCursor( waitCursor );
	    fdisk->getView()->defaultPartitionTable();
	    QApplication::restoreOverrideCursor();	
	    return true;
	  }
  }

  if ( w == partitionInfo )
 		return ( target->selected() != Target::Disk );

  if( w == installselect ){
		partitionInfo->setDone();
 #ifndef GBLI_DEVEL
    partitionInfo->doMounting();
 #endif
 	}

	if( w == videoCard && !installselect->withX )
		return FALSE;

  if( w == monitor && ( videoCard->defaultVGA16() || !installselect->withX ) )
    return FALSE;

  if( w == videoMode && ( videoCard->defaultVGA16() || !installselect->withX ) )
    return FALSE;

/*  if ( w == sound ){
    if( !LSound::hasSound() )// skip page if no sound could be configured
			return FALSE;
  }*/

  if ( w == grub) {
    grub->init();
    return true;
  }

  return QWizard::appropriate( w );
}


void LWizard::help()
{
		QMessageBox::information( this, tr("Help"),
			tr("In SOT we think that good interface does not need help.\n"
				"Please inform us if you know how to improve it's intuitivity.\n"
				"It is our intention to make things perfect."), tr("Ok") );
}


void LWizard::next()
{
    if( currentPage() == mouse ){
			versionLabel->hide();
  	}/*else if( currentPage() == upgradepage ){
			upgrade = upgradepage->type == UpgradePage::upgrade;
			if( upgrade ){
        if( !installselect )
					installselect = new InstallSelect( this );
				upgradepage->startUpgrade( installselect );
			}
		}  */
    QWizard::next();
}


void LWizard::layOutButtonRow( QHBoxLayout * l )
{
    l->addWidget( versionLabel );
    l->addWidget( progressLabel );
    l->addWidget( progressBar );
    QWizard::layOutButtonRow( l );
}

void LWizard::layOutTitleRow( QHBoxLayout * layout, const QString & title )
{
    titleLabel->setText( title );
		layout->addSpacing( 15 );
    layout->addWidget( titleLabel, 42 );
    layout->addWidget( titleLogo, 0 );
}



void LWizard::packageInstallationFinished()
{
    setFinishEnabled( finalpage, TRUE );
}


void LWizard::accept()
{
#ifdef GBLI_DEVEL
	qApp->exit();
	return;
#endif

		if (mouse)
       (void)::symlink( mouse->getPortName(), "/mnt/root/dev/mouse" );
    ::mkdir( "/mnt/root/home", 0775 );

    LEtcFiles etc( "/mnt/root/etc" );
    etc.writePasswdStuff( createusers->userName(), createusers->userLogin(), createusers->userPassword(),
			  createusers->password(), "/mnt/root/home", lang );
		if( installselect->withX ){
      etc.writeXConfig( videoCard, monitor, videoMode,
  		      mouse, keyboard, "/dev/mouse" );
		}
    etc.writeGpmConfig( mouse );
		etc.writeProfile( lang );
		etc.writeConfig( lang );

    // Save all network data
    netdata->save( "/mnt/root" );

    // chmod the swap spaces
    QStringList::Iterator dev;
    if( Swapper::swap_devices() )
      for (dev=Swapper::swap_devices()->begin(); dev != Swapper::swap_devices()->end(); ++dev) {
				QString rdev = QString("/mnt/root/")+*dev;
				::chmod(rdev.ascii(), 0600);
      }

		keyboard->finish( "/mnt/root/etc" );
		timezone->finish( "/mnt/root/etc" );
    grub->finish( installselect->withX );
		if( installselect->withX )
	    videoCard->finish();
//    sound->finish();

/*int pid = fork();
if( !pid ){
	int i = getdtablesize( );
	while( i-- >= 0 )
	  close( i );
	open( "/dev/null", O_RDWR );
	dup( 0 );
	dup( 1 );	
	exit( system( "/usr/bin/rpm --rebuilddb --root /mnt/root" ) );
} */


	// Now propose to create bootdisk
	grub->createBootdisk( installselect->withX );


//::waitpid( pid, NULL, 0 );

    qApp->exit();
}


void LWizard::reboot()
{
  QApplication::flushX();
  if ( QMessageBox::information( this, tr( "Control-Alt-Delete" ),
				   tr( "Are you sure you want to reboot?" ),
				   tr( "Reboot." ),
				   tr( "  Continue with install" ).simplifyWhiteSpace(),
				   QString::null,
				 1, 1 ) == 0 ) {
    ::sync();
    ::reboot(RB_AUTOBOOT);
  }
}


static int screendumpNumber = 0;

void LWizard::screendump()
{
    QPixmap p = QPixmap::grabWindow( topLevelWidget()->handle() );
    QString name;
    if( QFile( "/mnt/root/tmp" ).exists() )
      name.sprintf( "%s/tmp/gbli%02d.png", NEWROOT, ++screendumpNumber );
    else
      name.sprintf( "/tmp/gbli%02d.png", ++screendumpNumber );
    p.save( name, "PNG" );
}


void LWizard::keyPressEvent(QKeyEvent *e)
{
  if( e->key() == Key_Escape ){
    e->accept();
	} else {
    e->ignore();
    QWizard::keyPressEvent( e );
  }
}


void LWizard::version()
{
  if (versionLabel->isVisible())
    versionLabel->hide();
  else
    versionLabel->show();
}


void LWizard::xterm()
{
  // find an existing terminal
  unsigned int i, j, nwins, nsubwins;
  Window dw1, dw2, *wins, *subwins;
  XTextProperty attr;
  int count;
  char **list;
 
  XQueryTree(qt_xdisplay(), qt_xrootwin(), &dw1, &dw2, &wins, &nwins);
  for (i = 0; i < nwins; i++) {

    XQueryTree(qt_xdisplay(), wins[i], &dw1, &dw2, &subwins, &nsubwins);
    for (j = 0; j < nsubwins; j++) {

      XGetWMName(qt_xdisplay(), subwins[j], &attr);
      if (XTextPropertyToStringList(&attr, &list, &count)){
    	  if (count>0)
    	    if (strcmp(list[0], "System console") == 0){
        		XRaiseWindow(qt_xdisplay(), subwins[j]);
        		return;
 	      }
    	  XFreeStringList(list);
    	}
    }
    
    XFree((void *) subwins);
  }
  XFree((void *) wins); 

  // run a new terminal
  int pid = ::fork();
  if (pid == 0) {
    execl("./extrabin/xterm", "./extrabin/xterm", "-cm", "-dc", "-T", "System console", "-e", "/bin/bash", 0);
    exit(1);
  }
}













