/***************************************************************************
                               mountpoint.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "mountpoint.h"

MountPoint::MountPoint( const QString& mp , mount_size total )
{
    m = mp;
    t = total;
    reset();
}


void MountPoint::reset()
{
    uses.fill(0);
}


void MountPoint::add( uint id, mount_size kbytes )
{
     uses.at(id) += kbytes;
}

void MountPoint::add( const QValueList<uint> &ids, mount_size kbytes )
{
    for (QValueList<uint>::ConstIterator it = ids.begin(); it != ids.end(); it++)
			add( *it, kbytes );
}

mount_size MountPoint::used( uint id ) const
{
     return uses.at(id);
}


mount_size MountPoint::free( uint id ) const
{
  if (total()) {
    debug("total %d %d", (int)used(id), (int)(QMAX(used(id)*7 / 6, used(id) + 20000)) );
    return total() - QMAX(used(id)*7 / 6, used(id) + 20000);
  }
  return 0;
}

// I'm not quite sure about it, since this 'used' array looks strange,
// but nothing else to do in every case ;( Maddy
mount_size MountPoint::free() const
{
	mount_size usedsize = 0;
  for( QArray<mount_size>::ConstIterator it = uses.begin(); it != uses.end(); it++ )
		usedsize += (mount_size)*it;
	return total() - usedsize;
}

mount_size MountPoint::total() const
{
    return t;
}

MountPoint::~MountPoint()
{
}

QDict<QString> * MountPoints::mp = 0;
QDict<QString> * MountPoints::mpR = 0;
QDict<MountPoint> * MountPoints::lms = 0;

QString MountPoints::mountPoint( const QString& partition)
{
    if (!mp)
			return QString::null;
    QString* s = (*mp)[partition];
    if (s)
			return *s;
    return QString::null;
}

QString MountPoints::mountPartition( const QString& mount_point)
{
    if (!mpR)
			return QString::null;
    QString* s = (*mpR)[mount_point];
    if (s)
			return *s;
    return QString::null;
}


void MountPoints::setMountPoint( const QString& partition,
				  const QString& mount_point)
{
    QString p = mount_point;
    if (p.length() > 1 && p[ p.length() - 1] == '/') {
	// remove senseless '/' at the end
			p = p.left( p.length() - 1 );
    }
    
    if (!mp) {
			mp = new QDict<QString>;
			mp->setAutoDelete( TRUE );
    }
    if (!mpR) {
			mpR = new QDict<QString>;
			mpR->setAutoDelete( TRUE );
    }

    if ( (*mpR)[ p ] ) { // mount point already in use
			mp->remove( *((*mpR)[p]) );
			mpR->remove( mount_point );
    }
    if ( (*mp)[ partition ] ) { // partition already in use
			mpR->remove( *((*mp)[partition]) );
			mp->remove( partition );
    }

    mp->insert( partition, new QString( p) );
    mpR->insert( p, new QString( partition ) );
}


void MountPoints::clearMountPoint( const QString& partition )
{
    if (!mp || !mpR)
			return;

    if ( (*mp)[ partition ] ) { // partition already in use
      debug("clear mountpoint %s", (*mp)[partition]->ascii());
			mpR->remove( *((*mp)[partition]) );
			mp->remove( partition );
    }
}

void MountPoints::clear()
{
    if (mp)
			mp->clear();
    if (mpR)
			mpR->clear();
    if (lms)
      lms->clear();
}

bool MountPoints::newDirectory(const QString& name, mount_size size)
{
  // debug("new directory %s %d", name.ascii(), size);
  if( !lms ){
    lms = new QDict<MountPoint>();
    lms->setAutoDelete(true);
  }
  if( lms->find(name) ) { // if it's already known, we only get clashes
    return false;
  }
  lms->insert( name, new MountPoint(name, size * 1024) );
  return true;
}

MountPointList MountPoints::allDirectories()
{
    MountPointList tmp;
    if( !lms )
			return tmp;

    QDictIterator<MountPoint> it( *lms );
    while( it.current() ) {
			tmp.append( it.current() );
			++it;
    }
    return tmp;
}

void MountPoints::clearDirectories()
{
    if (lms)
			lms->clear();
}

void MountPoints::addToDirectory(const QString &path, mount_size size, const QValueList<uint> &ids)
{
  // debug("addToDirectory %s %d", path.ascii(), size);

    if (!lms)
			return;
    
    MountPoint *point;
    if( !path.isEmpty() )
			point = lms->find(path);
    else 
			point = lms->find("/");

    if( point ){
			point->add(ids, size);
	//	 debug("total %d %d", point->used(0) / 1024, point->used(0) / (0x100000));
    } else {
			addToDirectory( path.left(path.findRev('/') ), size, ids );
    }
   
}

MountPoint *MountPoints::directory(const QString& name) {
  if (!lms)
		return 0;
  return lms->find(name);
}



