/***************************************************************************
                          changepartitiondialog.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 1999 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "changepartitiondialog.h"
#include "diskitem.h"
#include "fdisk.h"
#include "mountpoint.h"

#include <qlayout.h>
#include <qbuttongroup.h>
#include <qfont.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qmessagebox.h>

/*  The change partition dialog is ....well.... used to change/edit a
  partition. It's a "private" auxiliary class of the partition view. */
ChangePartitionDialog::ChangePartitionDialog(QWidget* parent, const char *name)
    : QDialog( parent, name, TRUE )
{
		setFontPropagation( QWidget::AllChildren );
		setFont( QFont::defaultFont() );

    QVBoxLayout* vbox = new QVBoxLayout( this, 6 );
    QHBoxLayout* hbox;
    QLabel* label;

    // size settings

    groupSize = new QButtonGroup( tr("Partition Boundaries"), this );
    vbox->addWidget( groupSize );

    QVBoxLayout* gbvbox = new QVBoxLayout( groupSize, 20, 0 );//#### fixed rotten QLayout with QGroupBox

    hbox = new QHBoxLayout;
    gbvbox->addLayout( hbox );

    label = new QLabel( tr("Possible areas on disk"), groupSize );
    hbox->addWidget( label );
    hbox->addSpacing( 5 );
    freeSpaces = new QComboBox( FALSE, groupSize );
    connect( freeSpaces, SIGNAL( activated( const QString& ) ),
	     this, SLOT( freeSpaceComboActivated( const QString& ) ) );
    freeSpaces->insertItem("999999 - 999999");
    hbox->addWidget( freeSpaces );

    hbox->addStretch( 10 );

    gbvbox->addSpacing( 10 );

    hbox = new QHBoxLayout;
    gbvbox->addLayout( hbox );

    label = new QLabel( tr("Start"), groupSize );
    hbox->addWidget( label );
		hbox->addSpacing( 3 );
    startEdit = new QLineEdit( groupSize );
    startEdit->setValidator( new QIntValidator( groupSize ) );
    connect( startEdit, SIGNAL( textChanged(const QString&) ),
	     this, SLOT (lineEditChanged( const QString& ) ) );
    hbox->addWidget( startEdit );
    hbox->addSpacing( 12 );
    label = new QLabel( tr("End"), groupSize );
    hbox->addWidget( label );
		hbox->addSpacing( 3 );
    endEdit = new QLineEdit( groupSize );
    endEdit->setValidator( new QIntValidator(groupSize ) );
    connect( endEdit, SIGNAL( textChanged(const QString&) ),
	     this, SLOT (lineEditChanged( const QString& ) ) );
    hbox->addWidget( endEdit );
    hbox->addSpacing( 12 );
    sizeInfo = new QLabel( "10000TB", groupSize );
    QFont f = sizeInfo->font();
    f.setBold( TRUE );
    f.setItalic( TRUE );
    sizeInfo->setFont(f);
		sizeInfo->setFixedSize( sizeInfo->sizeHint() );
    hbox->addWidget( sizeInfo );

    // other settings

    groupSystem = new QButtonGroup( tr("System type"), this );
    vbox->addWidget( groupSystem );

    QVBoxLayout* vbox1 = new QVBoxLayout( groupSystem, 20, 0 );
    hbox = new QHBoxLayout;
		vbox1->addLayout( hbox );
    systemCombo = new QComboBox( FALSE, groupSystem );
    systemCombo->insertItem("DOS 16-bit FAT");
    connect( systemCombo, SIGNAL( activated( const QString& ) ),
	     this, SLOT( systemComboActivated( const QString& ) ) );
		hbox->addSpacing( 7 );
    hbox->addWidget( systemCombo, 5 );

		hbox->addSpacing( 15 );
    label = new QLabel( tr("Mount Point"), groupSystem );
    hbox->addWidget( label );
		hbox->addSpacing( 3 );

    mountPointEdit = new QComboBox( TRUE, groupSystem );
    mountPointEdit->insertItem( tr( "Unused" ) );
    mountPointEdit->insertItem( "/" );
    mountPointEdit->insertItem( "/usr" );
    mountPointEdit->insertItem( "/home" );
    mountPointEdit->insertItem( "/var" );
    mountPointEdit->insertItem( "/usr/local" );
    mountPointEdit->insertItem( "/tmp" );
    mountPointEdit->insertItem( "/opt" );
    mountPointEdit->insertItem( tr("<other>") );
    connect( mountPointEdit, SIGNAL( textChanged( const QString& ) ),
	     this, SLOT( mountPointComboActivated( const QString& ) ) );
    hbox->addWidget( mountPointEdit, 10);
		hbox->addSpacing( 7 );

    hbox = new QHBoxLayout;
		vbox1->addSpacing( 10 );
		vbox1->addLayout( hbox );
    bootableCheck = new QCheckBox( tr("Bootable")/* Partitions"*/, groupSystem );
		hbox->addSpacing( 10 );
    hbox->addWidget( bootableCheck );
		hbox->addStretch( 1 );
    formatCheck = new QCheckBox( /*tr("Partitions To */tr("Format"), groupSystem );
    hbox->addWidget( formatCheck );
		hbox->addStretch( 3 );

    // the buttons

    hbox = new QHBoxLayout( 6 );
    vbox->addLayout( hbox );

    hbox->addStretch( 10 );

    ok = new QPushButton( tr("Ok"), this );
    connect( ok, SIGNAL( clicked() ), this, SLOT( accept() ) );
    ok->setDefault( TRUE );
    hbox->addWidget( ok );

    cancel = new QPushButton( tr("Cancel"), this );
    connect( cancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    hbox->addWidget( cancel );

		setFont( QFont::defaultFont() );
}

ChangePartitionDialog::~ChangePartitionDialog()
{
}



/*
  This function is supposed to be called after the caller of the
  dialog set all communication parameters. It will update the
  graphical user interface and inform the dialog about the current
  disk.
 */
void ChangePartitionDialog::doSettings(DiskItem* disk_)
{
    disk = disk_;

    systemCombo->clear();
    systemCombo->insertItem( Fdisk::sysname(id) );
    if (id != 0x83)
			systemCombo->insertItem( Fdisk::sysname(0x83));
    if (id != 0x82)
			systemCombo->insertItem( Fdisk::sysname(0x82));
    if (id != 0x5 && !disk->hasExtendedPartition()) // only one extended partition possible
			systemCombo->insertItem( Fdisk::sysname(0x5));

    if (id != 0x6)
			systemCombo->insertItem( Fdisk::sysname(0x6) );

    QString n;
    n.setNum( disk->cylinderOfSector(start ) );
    startEdit->setText( n );
    n.setNum( disk->cylinderOfSector(start+size-1) );
    endEdit->setText( n );

    bootableCheck->setChecked( bootable );
    updateFormatCheck();

    if (id == 0x82) {
			// linux swap
			bootableCheck->setEnabled( FALSE );
			mountPointEdit->setCurrentItem( 0 );
			mountPointEdit->setEnabled( FALSE );
    }
    else {
			bootableCheck->setEnabled( TRUE );
			mountPointEdit->setEnabled( TRUE );
			QString m = MountPoints::mountPoint( name );
			if ( !m.isEmpty() )
	  	  mountPointEdit->setEditText( m );
			else
	  	  mountPointEdit->setCurrentItem( 0 ); // indicate unused
    }

    if (isNewPartition)
			freeSpaceComboActivated( freeSpaces->currentText() );
    else
			updateSizeInfo();

}

void ChangePartitionDialog::updateFormatCheck()
{
    bool mustBeFormated = Fdisk::isSwapType(id)  |  mountPoint() == "/" |   mountPoint() == "/usr"
      | curStart != start | curSize != size;


    if( Fdisk::isExtendedType(id) ){
			formatCheck->setEnabled( FALSE );
			formatCheck->setChecked( FALSE );
    } else if ( isNew || mustBeFormated ) {
			formatCheck->setChecked( TRUE );
//BestLinux - changes by Maddy
//			formatCheck->setEnabled( FALSE );
    } else {
			formatCheck->setEnabled( TRUE );
			formatCheck->setChecked( willBeFormated );
    }
    willBeFormated = formatCheck->isEnabled();
}

/*
  Recalculate the size of the partition and do the validation. If the
  partition is invalid, indicate this to the user and disable the "OK"
  button.
 */
void ChangePartitionDialog::updateSizeInfo()
{
  curStart = disk->sectorOfCylinder( startEdit->text().toInt() );
  if ( number > disk->nrPrimary || curStart == 0)
    curStart += disk->sectors; // DOS mode and some space for the boot manager

  if ( disk->cylinderOfSector( start ) == startEdit->text().toInt() )
    curStart = start; // fall back to the previous value

  curEnd = disk->sectorOfCylinder( endEdit->text().toInt() );

  curEnd += disk->heads * disk->sectors - 1; // align at exact _end_ of a cylinder

  if ( disk->cylinderOfSector( start + size -1 ) == endEdit->text().toInt() )
    curEnd = start + size - 1;

  curSize = curEnd - curStart + 1;

  bool valid = number>disk->nrPrimary ?
    disk->verifyFreeSpacesLogical( curStart, curEnd, number )
    :
    disk->verifyFreeSpacesPrimary( curStart, curEnd, number );

  QString n;
  if (valid ) {
    n.setNum( disk->megaByteOfSectors(curSize) );
    n += "MB";
  }
  else {
    n = tr("invalid");
  }
  sizeInfo->setText( n );

  ok->setEnabled( valid );
  updateFormatCheck();
}


/*
  reimplemented for internal purposes
 */
void ChangePartitionDialog::done( int result)
{
    if (result == QDialog::Accepted) {
    	bootable = bootableCheck->isChecked();
    	isNew |= (start != curStart) | (size != curSize );
    	start = curStart;
    	size = curSize;
    	if( Fdisk::isExtendedType(id) )
    	  willBeFormated = FALSE;
    	else
    	  willBeFormated = isNew | formatCheck->isChecked();
    }

    QDialog::done( result );
}

void ChangePartitionDialog::lineEditChanged( const QString&)
{
    if (Fdisk::isExtendedType( id ) )
			warnAboutSubpartitions();
    updateSizeInfo();
}


void ChangePartitionDialog::freeSpaceComboActivated( const QString& s)
{
    int minus = s.find('-');
    startEdit->setText( s.left(minus - 1) );
    endEdit->setText( s.right( s.length() - minus - 2) );
}



void ChangePartitionDialog::systemComboActivated( const QString& s)
{
    int oldId = id;
    id = Fdisk::systype( s );

    if (Fdisk::isExtendedType( oldId ) && !Fdisk::isExtendedType(id) ){
			warnAboutSubpartitions();
    }

    if (id == 0x82) { // Linux swap
    	bootableCheck->setChecked( FALSE );
    	bootableCheck->setEnabled( FALSE );
    	mountPointEdit->setCurrentItem( 0 );
    	mountPointEdit->setEnabled( FALSE );
    }
    else if (id == 0x5) { // extended
      bootableCheck->setChecked(FALSE);
      bootableCheck->setEnabled(FALSE);
      mountPointEdit->setCurrentItem(-1);
      mountPointEdit->setEnabled(FALSE);
    } else {
      bootableCheck->setEnabled( TRUE );
      mountPointEdit->setEnabled( TRUE );
    }

    updateFormatCheck();
}

void ChangePartitionDialog::mountPointComboActivated( const QString& mp)
{
  if (mp == tr("<other>"))
    {
      mountPointEdit->setEditText("");
      mountPointEdit->setFocus();
    }

  updateFormatCheck();
}


void ChangePartitionDialog::warnAboutSubpartitions()
{
    if ( hasSubpartitions && isVisible() ) {
			QMessageBox::information( this, tr("Logical partitions"),
	      tr("You are changing an extended partition. If you confirm the changes, \nall its subpartitions will be invalid and therefore removed.") );
			hasSubpartitions = FALSE;
    }
}

