/***************************************************************************
                              partitionview.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author	          	 : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "partitionview.h"
#include "partitionitem.h"
#include "partitioninfo.h"
#include "diskitem.h"
#include "changepartitiondialog.h"
#include "fdisk.h"
#include "mountpoint.h"
#include "swapper.h"

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/reboot.h>
#include <fcntl.h>

#include <qapplication.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>

PartitionView::PartitionView( QWidget *parent, const char *name )
    : QListView( parent, name)
{
    setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
		QFont font( QFont::defaultFont() );
 		font.setPointSize( 10 );
		font.setFamily( "times" );
    setFont( font );
    setSorting( -1 );
    view_invalidated = TRUE;

    int col;
    col = addColumn( tr("Device") );
    setColumnWidthMode( col, Maximum );
    setSorting( col , TRUE);

    col = addColumn( tr("Start") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignRight );
    setSorting( col, false );
    col = addColumn( tr("End") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignRight );
    col = addColumn( tr("Size") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignRight );
    col = addColumn( tr("System") );
    setColumnWidthMode( col, Maximum );
    col = addColumn( tr("Bootable") );
    setColumnWidthMode( col, Manual );
    setColumnWidth( col, 30 );
    setColumnAlignment( col, AlignCenter );
    col = addColumn( tr("Mount Point") );
    setColumnWidthMode( col, Maximum );
    col = addColumn( tr("Format") );
    setColumnWidthMode( col, Maximum );
//    setColumnAlignment( col, AlignCenter );

    somethingWritten = FALSE;

    connect( this, SIGNAL (selectionChanged( QListViewItem* ) ),
	     this, SLOT (selectionChanged( QListViewItem* ) ) );

    partitionDialog = new ChangePartitionDialog( this );

}


PartitionView::~PartitionView()
{
}


/*!
  reread the partition table if the view is invalidated
 */
void PartitionView::checkState()
{
    if (viewInvalidated()) {
			QApplication::setOverrideCursor( waitCursor );
			rereadPartitionTable();
			QApplication::restoreOverrideCursor();
    }
}


PartitionItem* PartitionView::currentPartition()
{
    if (!current || current->isDisk() )
			return 0;
    return (PartitionItem*) current;
}

DiskItem* PartitionView::currentDisk()
{
    if (!current )
			return 0;
    if( currentPartition() )
			return currentPartition()->diskItem;
    return (DiskItem*) current;
}




/*
  invalidates the view. That means the table will be updated when the
  partition view is shown the next time.
 */
void PartitionView::invalidateView()
{
    view_invalidated = TRUE;
}


/*
  updates the view
 */

void PartitionView::updateView()
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    p->updateView();
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
}

void PartitionView::editCurrent()
{
    if (!currentPartition())
			return;
    if (currentDisk()->edit( currentPartition() )) {
			emit changed( TRUE );
			emit notOriginal( TRUE );
    }
}

void PartitionView::addLogicalPartition()
{
    if (!currentDisk())
			return;
    if (currentDisk()->addLogicalPartition() ) {
			emit changed( TRUE );
			emit notOriginal( TRUE );
    }
}

void PartitionView::removeOrClearCurrent()
{
    if (!currentPartition())
			return;
    currentDisk()->removeOrClear( currentPartition() );


    emit changed( TRUE );
    emit notOriginal( TRUE );

}


/*
  Writes the modified partition tables back onto the hardisk.
*/
void PartitionView::writePartitionTable()
{
    QApplication::setOverrideCursor( waitCursor );

    // stop swapping, we might swap on this disk!
    Swapper::swapOff();

    QListViewItem*  i = firstChild();
    while ( i ) {
			((DiskItem*)i)->writePartitionTable();
			i = i->nextSibling();
    }

    somethingWritten = TRUE;
    emit changed( FALSE );
    QApplication::restoreOverrideCursor();
}


/*
  Undos everything to the state when the partition view was created.
 */
void PartitionView::defaultPartitionTable()
{
    emit currentAvailable( FALSE );

    MountPoints::clear(); // there were no mount points when he first came here

    QListViewItem*  i = firstChild();
    while ( i ) {
			( (DiskItem*)i )->defaultPartitionTable();
			i = i->nextSibling();
    }

    emit changed( somethingWritten );
    emit notOriginal( FALSE );

    updateView();
}

void PartitionView::rereadPartitionTable()
{
    view_invalidated = FALSE;
		
		PartitionInfo::unMounting();// lets try to make it this way (Maddy)

    clear();
    // put all available disks into the partition view
    QStringList availableDisks = Fdisk::availableDisks();
    QStringList::Iterator disk;
    for (disk = availableDisks.begin(); disk != availableDisks.end(); ++disk) {
			(new DiskItem( *disk, this, partitionDialog ))->setOpen( TRUE );
    }
    emit diskCheckMaximum( allPartitions().count() );

    int n = 1;
    for (DiskItem* i =  (DiskItem*)firstChild(); i; i =  (DiskItem*) i->nextSibling() ) {
    	for (PartitionItem* p =  (PartitionItem*)i->firstChild(); p; p =  (PartitionItem*) p->nextSibling() ) {
    	    p->checkForFilesystem();
    	    emit diskCheckProgress( n++ );
    	    for (PartitionItem* lp =  (PartitionItem*)p->firstChild(); lp; lp =  (PartitionItem*) lp->nextSibling() ) {
    				lp->checkForFilesystem();
    				emit diskCheckProgress( n++ );
    	    }
    	}
    }

    somethingWritten = FALSE;
    emit changed( FALSE );
    emit notOriginal( FALSE );
    Fdisk::startup = FALSE; // end initialization
}



void PartitionView::selectionChanged( QListViewItem * item)
{
    current = (DiskOrPartitionItem*) item;
    if (currentPartition() ) {
	emit currentAvailable( TRUE );
    }
    else {
	emit currentAvailable( FALSE );
    }
    emit diskAvailable( currentDisk() != 0 );
}




/*
  Returns a stringlist of all currently available partitions in this partition view
 */
QStringList PartitionView::allPartitions()
{
    QStringList result;
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    result.append( p->name() );
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return result;
}


/*
  Returns a stringlist of all currently available swap partitions in
  this partition view
 */
QStringList PartitionView::allSwapPartitions()
{
    QStringList result;
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    if ( p->isSwap() )
		    		result.append( p->name() );
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return result;
}


/*
  Returns whether some of the disks already contain at least one Linux
  partition.
 */
bool PartitionView::hasLinuxPartitions()
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while( p ) {
    	    if ( p->id == 0x83 )
		    		return TRUE;
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

/*
  Returns a pointer to the specified partition item
 */
PartitionItem* PartitionView::partition( const QString& name )
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    if ( p->name() == name )
		    		return p;
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return 0;
}

/*
  Returns the size of the specified partition in MB
 */
double PartitionView::sizeOfPartition(const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
			return p->sizeInMB();
    return 0;
}

/*!
  Returns the size of the specified partition in MB as QString
 */
QString PartitionView::sizeOfPartitionAsString(const QString& name)
{
    QString result;
    result.setNum( sizeOfPartition( name) );
    return result + "MB";
}

/*
  Returns the id of the specified partition (system type)
 */
int PartitionView::idOfPartition(const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
			return p->id;
    return 0;
}

/*
  Returns whether the specified partition is below the first 1024
  cylinders ( which makes it possible for current PC bioses to boot a
  kernel from such a partition).

 */
bool PartitionView::partitionBelow1024Cylinders(const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
      return p->cylinderOfSector( p->end() ) <= 1024;
    return FALSE;
}


/*
  Returns whether the specfied partition is bootable
 */
bool PartitionView::bootable( const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
			return p->bootable;
    return FALSE;
}


/*!
  Sets the bootable flag of the specfied partition
 */
void PartitionView::setBootable( const QString& name, bool value)
{
    PartitionItem* p = partition( name );
    if ( p  && p-> bootable != value ) {
			p->bootable = value; // set the partition's bootable flag
			p->diskItem->invalidateDisk(); // inform the disk that it is dirty now
    }
}



/*
  Returns a pointer to the specified disk item
 */
DiskItem* PartitionView::disk( const QString& name )
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk && disk->disk != name) {
			disk = (DiskItem*) disk->nextSibling();
    }
    return disk;
}


/*!
  Returns the size of the specified disk
 */
double PartitionView::capacityOfDisk(const QString& name)
{
    DiskItem* d = disk( name );
    if (!d)
			return 0;
    return  d->capacity();

}

/*!
  Returns the size of the specified disk as a string
 */
QString PartitionView::capacityOfDiskAsString(const QString& name)
{
    QString result;
    result.setNum( capacityOfDisk( name) );
    return result + "MB";
}

bool PartitionView::hasLinuxPartitions(const QString& name)
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	if ( disk->disk == name) {
    	    int n = 1;
    	    PartitionItem* p = disk->partition( n );
    	    while ( p ) {
        		if ( p->id == 0x83 )
       		    return TRUE;
        		n++;
        		p = disk->partition( n );
    	    }
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

bool PartitionView::hasDosWindowsPartitions(const QString& name)
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	if ( disk->disk == name) {
    	    int n = 1;
    	    PartitionItem* p = disk->partition( n );
    	    while (p ) {
        		if ( p->id == 0x01 ||
        		     p->id == 0x04 ||		
        		     p->id == 0x11 ||		
        		     p->id == 0x14 ||		
        		     p->id == 0x16 ||		
        		     p->id == 0x24 ||		
        		     p->id == 0xc1 ||		
        		     p->id == 0xc4 ||		
        		     p->id == 0xc6 ||		
        		     p->id == 0xc4 ||		
        		     p->id == 0x06 ||		
        		     p->id == 0x0b ||		
        		     p->id == 0x0e ||		
        		     p->id == 0x0f
        		     )
        		    return TRUE;
        		n++;
        		p = disk->partition( n );
    	    }
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

bool PartitionView::hasPartitions(const QString& name)
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	if ( disk->disk == name) {
    	    int n = 1;
    	    PartitionItem* p = disk->partition( n );
    	    while (p ) {
        		if ( p->id != 0 )
        		    return TRUE;
        		n++;
        		p = disk->partition( n );
    	    }
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

void PartitionView::formatPartition( QString device )
{
    QString result;
    QApplication::flushX();
    int fd[2];
    ::pipe(fd);
    int pid = ::fork();
    if (pid == 0) {
    	::close( fd[0] );	
    	
    	if (fd[1] != STDOUT_FILENO) {
    	    dup2( fd[1], STDOUT_FILENO);
    	    ::close( fd[1] );
    	}
    	
    	int id = idOfPartition( device );
    	if( id == 0x83 ){
  	    if( device.left(8) == "/dev/rd/" )
  	      execl("./extrabin/mkfs.ext2", "./extrabin/mkfs.ext2", device.ascii(), 0);
  	    else
  	      execl("./extrabin/mkfs.ext2", "./extrabin/mkfs.ext2", device.ascii(), 0);
   	  }else if( id == 0x6 ){// FAT
					char buf[128];
					sprintf( buf, "dd if=/dev/zero of=%s bs=512 count=1", device.ascii() );
					system( buf );
    	    execl("./extrabin/mkfs.msdos", "./extrabin/mkfs.msdos", "-c", device.ascii(), 0);
    	}else if( Fdisk::isSwapType( id ) ){
    	    execl("./extrabin/mkswap", "./extrabin/mkswap", device.ascii(), 0);
    	}
    	exit(1);
    }
printf( "hasPartitions pid: %d\n", pid );
    ::close( fd[1] );

    QFile f;
    if ( f.open(IO_ReadOnly, fd[0]) ) {    // file opened successfully
    	QTextStream t( &f );	    // use a text stream
    	result = t.read();
    	::close( fd[0] );
    }
	
    ::waitpid(pid, NULL, 0);

    if ( Fdisk::isSwapType( idOfPartition( device ) ) )
			Swapper::swapOn( device );
}


