/***************************************************************************
                               upgradepage.cpp
                             -------------------                                         
    begin                : Tue Jan 4 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "upgradepage.h"
#include "log.h"
#include "installselect.h"
#include "windows.h"

#include <qlayout.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include <mntent.h>
#include <sys/swap.h>

UpgradePage::UpgradePage( QWidget * parent, const char * name )
    : QWidget( parent, name ), type( install )
{
  gb = new QButtonGroup( this );
	gb->setExclusive( true );

  QHBoxLayout * lh = new QHBoxLayout( this, 0, 6 );

  QVBoxLayout * l = new QVBoxLayout( 6 );
	lh->addStretch( 5 );
  lh->addLayout( l, 2 );
	lh->addStretch( 5 );

  QHBoxLayout* lrh = new QHBoxLayout();
  l->addStretch();
  l->addLayout( lrh );
  l->addStretch();

  QVBoxLayout *lr = new QVBoxLayout();
  lrh->addStretch( 1 );
  lrh->addLayout( lr, 5 );
  lrh->addStretch( 1 );
	
	rbInstall = new QRadioButton( tr( "Install"), this );
  lr->addWidget( rbInstall, 3, AlignTop + AlignLeft );
  gb->insert( rbInstall, install );
	rbUpgrade = new QRadioButton( tr( "Upgrade"), this );
  lr->addWidget( rbUpgrade, 3, AlignTop + AlignLeft );
  gb->insert( rbUpgrade, upgrade );

  connect( gb, SIGNAL(clicked(int)), this, SLOT(setSelected(int)) );

	rbInstall->setChecked( true );
}

UpgradePage::~UpgradePage() {
}

void UpgradePage::setSelected( int a_i ) {
	type = a_i;
}

/*
"A complete log of your upgrade will be in /tmp/upgrade.log when the upgrade "
"is finished. After rebooting, please read it to ensure configuration files "
"are properly updated."

"Rebuilding"

"Rebuilding RPM database..."

"Rebuild of RPM database failed. You may be out of disk space?"

"Installation Path"

"Install"
"Upgrade"

""
"Would you like to install a new system or upgrade a system which already "
"contains Best Linux 2.0 or later?"

"Converting RPM database..."

"Finding packages to upgrade..."

"You don't have any Linux partitions. You can't upgrade this system!"

"Upgrade Packages"

"The packages you have installed, and any other packages which are needed to "
"satisfy their dependencies, have been selected for installation. Would you "
"like to customize the set of packages that will be upgraded?"

*/


void UpgradePage::compareFileList( int availFileCount, char **availFiles,
			    int installedFileCount, char **installedFiles )
{
    int installedX, availX, rc;

    availX = 0;
    installedX = 0;
    while( installedX < installedFileCount ) {
    	if( availX == availFileCount ) {
  	    /* All the rest have moved */
  	    /* printf("=> %s\n", installedFiles[installedX]); */
  	    if( strncmp( installedFiles[installedX], "/etc/rc.d/", 10 ) )
	    		htAddToTable( ht, installedFiles[installedX] );
  	    installedX++;
    	}else{
  	    rc = strcmp(availFiles[availX], installedFiles[installedX]);
  	    if (rc > 0) {
  	  		/* Avail > Installed -- file has moved */
    			/* printf("=> %s\n", installedFiles[installedX]); */
	    		if (strncmp(installedFiles[installedX], "/etc/rc.d/", 10))
    		    htAddToTable(ht, installedFiles[installedX]);
	    		installedX++;
   	    } else if (rc < 0) {
  	  		/* Avail < Installed -- avail has some new files */
    			availX++;
    	  } else {
    			/* Files are equal -- file not moved */
    			availX++;
    			installedX++;
  	    }
    	}
    }
}

static int pkgCompare_( const void * first, const void * second )
{
    packageInfo ** a = (packageInfo **)first;
    packageInfo ** b = (packageInfo **)second;

    // put packages w/o names at the end
    if (!(*a)->name) return 1;
    if (!(*b)->name) return -1;

    return strcasecmp( (*a)->name, (*b)->name );
}

void UpgradePage::addLostFiles()
{
    int num;
    Header h;
    char *name;
    struct packageInfo **pack;
    struct packageInfo key;
    struct packageInfo *keyaddr = &key;
    char **installedFiles;
    int installedFileCount;

    num = rpmdbFirstRecNum(db);
    while (num) {
    	h = rpmdbGetRecord(db, num);
    	headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
    	if (name && !strcmp(name, "metroess")) {
  	    /* metro was removed from 5.1, but leave it if it's already
  	       installed */
  	    headerFree(h);
  	    num = rpmdbNextRecNum(db, num);
  	    continue;
    	}
    	key.name = name;
    	
    	pack = (packageInfo **)bsearch( &keyaddr, psp->packages, psp->numPackages,
    		       sizeof( *psp->packages ),
							/*(void *)*/pkgCompare_ );
    	if( !pack ) {
  	    if( headerGetEntry(h, RPMTAG_FILENAMES, NULL,
  			  (void **) &installedFiles, &installedFileCount ) ) {
		  		compareFileList( 0, NULL, installedFileCount,
  					installedFiles );
  				free( installedFiles );
  	    }
    	}
    	
    	headerFree(h);
    	num = rpmdbNextRecNum(db, num);
    }
}

int UpgradePage::findPackagesWithObsoletes()
{
    dbiIndexSet matches;
    int rc, count, obsoletesCount;
    struct packageInfo **pip;
    char **obsoletes;

    count = psp->numPackages;
    pip = psp->packages;
    while (count--) {
    	if ((*pip)->selected) {
    	    pip++;
    	    continue;
    	}

    	if (headerGetEntry((*pip)->h, RPMTAG_OBSOLETES, NULL,
    		       (void **) &obsoletes, &obsoletesCount)) {
   	    while (obsoletesCount--) {
      		rc = rpmdbFindPackage(db, obsoletes[obsoletesCount], &matches);
      		if (!rc) {
    		    if (matches.count) {
        			(*pip)->selected = 1;
        			dbiFreeIndexRecord(matches);
        			break;
    		    }

    		    dbiFreeIndexRecord(matches);
      		}
   	    }

   	    free(obsoletes);
    	}

    	pip++;
    }

    return 0;
}

static void errorFunction(void)
{
}

int UpgradePage::findUpgradePackages()
{
    int skipThis;
    Header h, installedHeader;
    char *name, *version, *release;
    dbiIndexSet matches;
    int rc, i, count;
    char **installedFiles, **availFiles;
    int installedFileCount, availFileCount;
    struct packageInfo **pip;

    count = psp->numPackages;
    pip = psp->packages;
    while (count--) {
    	h = (*pip)->h;
    	name = version = release = NULL;
    	headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
    	headerGetEntry(h, RPMTAG_VERSION, NULL, (void **) &version, NULL);
    	headerGetEntry(h, RPMTAG_RELEASE, NULL, (void **) &release, NULL);
    	if( !(name && version && release) ) {
  	    /* bum header */
  	    Log::logMain( 32 , "Failed with bad header" );
  	    return(INST_ERROR);
    	}

    	/*printf("Avail: %s-%s-%s\n", name, version, release);*/
    	rc = rpmdbFindPackage(db, name, &matches);

    	if (rc == 0) {
  	    skipThis = 0;
  	    rpmErrorSetCallback(errorFunction);
  	    for (i = 0; i < matches.count; i++) {
      		installedHeader =
     		    rpmdbGetRecord(db, matches.recs[i].recOffset);
      		if (rpmVersionCompare(installedHeader, h) >= 0) {
    		    /* already have a newer version installed */
    		    /*printf("Already have newer version\n");*/
    		    skipThis = 1;
    		    headerFree(installedHeader);
    		    break;
      		}
      		headerFree(installedHeader);
  	    }
  	    rpmErrorSetCallback(NULL);
  	    if (! skipThis) {
	  		/*printf("No newer version installed\n");*/
  	    }
    	} else {
  	    skipThis = 1;
  	    /*printf("Not installed\n");*/
    	}
    	
    	if (skipThis) {
  	    /*printf("DO NOT INSTALL\n");*/
    	} else {
  	    /*printf("UPGRADE\n");*/
  	    (*pip)->selected = 1;

  	    if (!headerGetEntry(h, RPMTAG_FILENAMES, NULL,
  			  (void **) &availFiles, &availFileCount)) {
      		availFiles = NULL;
      		availFileCount = 0;
  	    }

  	    for (i = 0; i < matches.count; i++) {
      		/* Compare the file lists */
      		installedHeader =
     		    rpmdbGetRecord(db, matches.recs[i].recOffset);
      		if( !headerGetEntry(installedHeader, RPMTAG_FILENAMES, NULL,
  			      (void **) &installedFiles,
  			      &installedFileCount) ) {
    		    installedFiles = NULL;
    		    installedFileCount = 0;
      		}

      		compareFileList( availFileCount, availFiles,
    				installedFileCount, installedFiles );

      		if( installedFiles )
    		    free( installedFiles );
      		headerFree( installedHeader );
  	    }

  	    if (availFiles)
		  		free(availFiles);
    	}

    	if (rc == 0)
  	    dbiFreeIndexRecord(matches);

    	/*printf("\n\n");*/

    	pip++;
    }

    return 0;
}

int UpgradePage::removeMovedFilesAlreadyHandled()
{
    char *name;
    int i, count;
    Header h;
    char **availFiles;
    int availFileCount;
    char *file;
    struct packageInfo **pip;

    count = psp->numPackages;
    pip = psp->packages;
    while (count--) {
    	h = (*pip)->h;
    	if ((*pip)->selected) {
  	    name = NULL;
  	    headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);

  	    if (!headerGetEntry(h, RPMTAG_FILENAMES, NULL,
  			  (void **) &availFiles, &availFileCount)) {
		  		availFiles = NULL;
  				availFileCount = 0;
  	    }

  	    for (i = 0; i < availFileCount; i++) {
      		if ((file = htInTable(ht, availFiles[i]))) {
      		    *file = '\0';
      		    /*printf("File already in %s: %s\n", name, availFiles[i]);*/
      		    break;
      		}
  	    }
  	    if( availFiles )
		  		free(availFiles);
    	}

    	pip++;
    }

    return 0;
}

int UpgradePage::findPackagesWithRelocatedFiles()
{
    char *name;
    int i, count;
    Header h;
    char **availFiles;
    int availFileCount;
    char *file;
    struct packageInfo **pip;

    count = psp->numPackages;
    pip = psp->packages;
    while (count--) {
    	h = (*pip)->h;
    	if (! (*pip)->selected) {
  	    name = NULL;
  	    headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);

  	    availFiles = NULL;
  	    availFileCount = 0;
  	    if (headerGetEntry(h, RPMTAG_FILENAMES, NULL,
  			 (void **) &availFiles, &availFileCount)) {
      		for (i = 0; i < availFileCount; i++) {
    		    if ((file = htInTable(ht, availFiles[i]))) {
        			*file = '\0';
        			/*printf("Found file in %s: %s\n", name,
        			  availFiles[i]);*/
        			(*pip)->selected = 1;
        			break;
    		    }
      		}
		  		free(availFiles);
		    }
			}

			pip++;
    }

    return 0;
}

void UpgradePage::printCount()
{
    int i, upgradeCount;
    struct packageInfo **pip;

    upgradeCount = 0;
    pip = psp->packages;
    i = psp->numPackages;
    while( i-- ){
     	if ((*pip)->selected) {
   	    upgradeCount++;
     	}
     	pip++;
    }
    Log::logMain( 33, "marked %d packages for upgrade", upgradeCount );
}

int UpgradePage::unmarkPackagesAlreadyInstalled()
{
    dbiIndexSet matches;
    Header h, installedHeader;
    char *name, *version, *release;
    struct packageInfo **pip;
    int count, rc, i;

    count = psp->numPackages;
    pip = psp->packages;
    while (count--) {
    	if ((*pip)->selected) {
  	    h = (*pip)->h;
  	    /* If this package is already installed, don't bother */
  	    name = version = release = NULL;
  	    headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
  	    headerGetEntry(h, RPMTAG_VERSION, NULL, (void **) &version, NULL);
  	    headerGetEntry(h, RPMTAG_RELEASE, NULL, (void **) &release, NULL);
  	    if (! (name && version && release)) {
      		/* bum header */
      		Log::logMain( 32, "Failed with bad header" );
      		return(INST_ERROR);
  	    }
  	    rc = rpmdbFindPackage(db, name, &matches);
  	    if( rc == 0 ) {
      		rpmErrorSetCallback(errorFunction);
      		for( i = 0; i < matches.count; i++ ){
    		    installedHeader =
	      			rpmdbGetRecord(db, matches.recs[i].recOffset);
     		    if (rpmVersionCompare( installedHeader, h ) >= 0) {
        			/* already have a newer version installed */
        			/*printf("Already have newer version\n");*/
        			(*pip)->selected = 0;
        			headerFree(installedHeader);
        			break;
    		    }
    		    headerFree(installedHeader);
      		}
      		rpmErrorSetCallback(NULL);
      		dbiFreeIndexRecord(matches);
  	    }
    	}

    	pip++;
    }

    return 0;
}
	
static void emptyErrorCallback(void) {
}

int UpgradePage::startUpgrade( InstallSelect *a_install )
{
		int res = a_install->loadData();
		if( res != INST_OK ) return res;

		psp = &(a_install->ps);
    rpmErrorCallBackType old;

    Log::logMain( 34, ("ugFindUpgradePackages() ...") );

    rpmReadConfigFiles( NULL, NULL );

    rpmSetVerbosity( RPMMESS_FATALERROR );
    old = rpmErrorSetCallback( emptyErrorCallback );

    if( rpmdbOpenForTraversal( NEWROOT, &db ) ) {
    	Log::logMain( 0, "failed opening %s/var/lib/rpm/packages.rpm",
    		     NEWROOT );
    	return INST_ERROR;
    }

    rpmErrorSetCallback( old );
    rpmSetVerbosity( RPMMESS_NORMAL );

    ht = htNewTable( 1103 );

    /* For all packages that are installed, if there is no package       */
    /* available by that name, add the package's files to the hash table */
    addLostFiles();
    Log::logMain( 34, ("added lost files") );
    printCount();

    /* Find packages that are new, and mark them in installThisPackage,  */
    /* updating availPkgs with the count.  Also add files to the hash   */
    /* table that do not exist in the new package - they may have moved */
    if( findUpgradePackages() ){
			rpmdbClose( db );
			return INST_ERROR;
    }
    Log::logMain( 34, ("found basic packages to upgrade") );
    printCount();
    /*hash_stats(hashTable);*/

    /* Remove any files that were added to the hash table that are in */
    /* some other package marked for upgrade.                         */
    removeMovedFilesAlreadyHandled();
    Log::logMain( 34, ("removed extra files which have moved") );
    printCount();

    findPackagesWithRelocatedFiles();
    Log::logMain( 34, ("found packages with relocated files") );
    printCount();

    findPackagesWithObsoletes();
    Log::logMain( 34, ("found packages that obsolete installed packages") );
    printCount();

    unmarkPackagesAlreadyInstalled();
    Log::logMain( 34, ("unmarked packages already installed") );
    printCount();

    htFreeHashTable( ht );

    /*printMemStats("Done");*/

    rpmdbClose(db);

		QTimer::singleShot( 200, a_install, SLOT(runInstall()) );

    return 0;
}

int UpgradePage::mountFileSystem()
{
    FILE * f;
    char buf[128];
		bool mountedRoot = false;

		struct mntent *entry;

//    umountFilesystems(finalFstab); ??? maybe TODO

#ifdef GBLI_DEVEL
		sprintf( buf, "/etc/fstab" );
#else
		sprintf( buf, "%s/etc/fstab", NEWROOT );
#endif
		f = setmntent( (const char *)buf, "r" );

    if( !f ){
			errorWindow( this, tr("Cannot read /mnt/etc/fstab: %s") );
			Log::logMain( 32, "Cannot read /mnt/etc/fstab: %s" );
			return INST_ERROR;
    }

		while( (entry = getmntent(f)) ){
    	if( strncmp( entry->mnt_fsname, "/dev/", 5 ) ) continue;
    	if( strstr( entry->mnt_opts, "noauto" ) ) continue;

      if( strncmp( entry->mnt_fsname, "/dev/hd", 7 ) &&
	        strncmp( entry->mnt_fsname, "/dev/sd", 7 ) &&
  	      strncmp( entry->mnt_fsname, "/dev/rd/", 8 ) &&
	  	 		strncmp( entry->mnt_fsname, "/dev/ida/", 9 ) )
	    	continue;

			if( !strcmp( entry->mnt_type, "ext2" ) ||
					!strcmp( entry->mnt_type, "msdos" ) ){
				if( !mount( entry->mnt_fsname, entry->mnt_dir, entry->mnt_type,
					0, NULL ) ){
					// mount failed error
          continue;
				}
			}else if( !strcmp( entry->mnt_type, "swap" ) ){
				if( !swapon( entry->mnt_fsname, 0 ) ){
					Log::logMain( 32, "Swap mount failed on %s", entry->mnt_fsname );
					continue;
				}
			}
			if( !strcmp( entry->mnt_dir, "/" ) )
				mountedRoot = true;
		}

		if( !mountedRoot ){
			errorWindow( this, tr("You don't have any Linux partitions. You can't upgrade this system!") );
			Log::logMain( 32, "You don't have any Linux partitions. You can't upgrade this system!" );
			endmntent( f );
			return INST_ERROR;
		}

		endmntent( f );

    return INST_OK;
}