#include <alloca.h>
#include <newt.h>
#include <popt.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/mount.h>

#include "devices.h"
#include "install.h"
#include "intl.h"
#include "grub.h"
#include "log.h"
#include "run.h"
#include "windows.h"
#include "smp.h"

#define KERNEL_IMAGE "vmlinuz-"
#define KERNEL_PURE_IMAGE "/boot/vmlinuz"
#define KERNEL_SMPIMAGE "vmlinuz-"

#define LILOKERNEL_IMAGE "/boot/vmlinuz-%s"
#define LILOKERNEL_SMPIMAGE "/boot/vmlinuz-%ssmp"

#ifdef __sparc__
#define GRUB_BINARY "/mnt/sbin/silo"
#else
#define GRUB_BINARY "/mnt/sbin/grub"
#endif

int link_file( int a_iSMP, char *a_szKernelPrefix, char *a_szKernelVersion, char *a_szFileName, char *a_szMapAs ) {
char szPathName[ 1024 ];
int iPathLen;
char szLinkPath[ 1024 ];
//char szLinkedAs[ 1024 ];

	if  ( chdir( a_szKernelPrefix ) ) return -1;

	iPathLen = readlink(  a_szMapAs, szPathName, 1024 );
	 //If iPathLen == -1 file not found or it`s not readable use default file name
	if ( iPathLen != -1 ) {
		//Check if link contain perfect SMP support & look`s like current version
		szPathName[ iPathLen ] = 0;
		//Check first version number
		if ( strstr( szPathName, a_szKernelVersion ) ) {
			if ( strstr( szPathName, "smp" ) ) {
				if ( a_iSMP ) return 0;
			}
		}
		if ( unlink( a_szMapAs ) ) return errno* -1;
	}
	sprintf( szLinkPath, "%s-%s", a_szFileName, a_szKernelVersion );
	if ( a_iSMP ) strcat( szLinkPath, "smp" );
	
	if ( symlink( szLinkPath, a_szMapAs ) ) return errno * -1;

	return 0;
}

int link_boot_files( int a_iSMP, char *a_szKernelPrefix, char *a_szKernelVersion ) {
int iRetStat = 0;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "System.map", "System.map" );
	if ( iRetStat < 0 ) return iRetStat;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "module-info", "module-info" );
	if ( iRetStat < 0 ) return iRetStat;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "vmlinuz", "vmlinuz" );
	if ( iRetStat < 0 ) return iRetStat;

	return 0;
}

int GoodOldLILOConfWrite(  char * prefix, char * dev, char * rootdev,
			 char * bootdev, struct partitionTable table,
			 char * append, char * kernelVersion,
			 char * hdname, int issmp,
			 int usesmpInitrd, char *relsmpinitrdImage,
			 int useInitrd, char *relinitrdImage  ) {
 FILE * f;
 int pass;
 int i;
 char filename[100];
 static int LilofirstTime = 1;

    /* why not? */
    if (LilofirstTime) {
	rename("/mnt/etc/lilo.conf", "/mnt/etc/lilo.conf.rpmsave");
	rename("/mnt/etc/silo.conf", "/mnt/etc/silo.conf.rpmsave");
	LilofirstTime = 0;
    }
    sprintf(filename, "%s/lilo.conf", prefix);

    f = fopen(filename, "w");
    if (!f) {
	errorWindow("cannot create lilo config file: %s");
	return INST_ERROR;
    }

    logMessage("writing lilo config to %s", filename);

    fprintf(f, "boot=/dev/%s\n", dev);
    fprintf(f, "map=/boot/map\n");
    fprintf(f, "install=/boot/boot.b\n");
    fprintf(f, "prompt\n");
//    if (linear) fprintf(f, "linear\n");
    fprintf(f, "timeout=50\n");
    for (pass = 0; pass < 2; pass++) {
	for (i = 0; i < table.count; i++) {
	    if (!table.parts[i].bootLabel) continue;
	    if (pass == 0 && !table.parts[i].defaultBoot) continue;
	    if (pass == 1 && table.parts[i].defaultBoot) continue;

	    if (table.parts[i].type == PART_EXT2) {
          		if (issmp) {
		    fprintf(f, "image=" LILOKERNEL_SMPIMAGE "\n", kernelVersion);
		    fprintf(f, "\tlabel=%s\n", table.parts[i].bootLabel);

	            /* remap /boot to root=/, but make sure we set up booting
		       from old root partitions as well */
		    if (strcmp(bootdev, table.parts[i].device))
			fprintf(f, "\troot=/dev/%s\n", table.parts[i].device);
		    else
			fprintf(f, "\troot=/dev/%s\n", rootdev);
		    if (usesmpInitrd)
		        fprintf(f, "\tinitrd=%s\n", relsmpinitrdImage);
		    if (append) fprintf(f, "\tappend=\"%s\"\n", append);
			fprintf(f, "\tread-only\n");
	          	}
		fprintf(f, "image=" LILOKERNEL_IMAGE "\n", kernelVersion);
		if (issmp)
		    fprintf(f, "\tlabel=linux-up\n");
		else
		    fprintf(f, "\tlabel=%s\n", table.parts[i].bootLabel);

		/* remap /boot to root=/, but make sure we set up booting
		   from old root partitions as well */
		if (strcmp(bootdev, table.parts[i].device))
		    fprintf(f, "\troot=/dev/%s\n", table.parts[i].device);
		else
		    fprintf(f, "\troot=/dev/%s\n", rootdev);
		if (useInitrd)
		    fprintf(f, "\tinitrd=%s\n", relinitrdImage);
		if (append) fprintf(f, "\tappend=\"%s\"\n", append);
		fprintf(f, "\tread-only\n");

	    } else {
		fprintf(f, "other=/dev/%s\n", table.parts[i].device);
		fprintf(f, "\tlabel=%s\n", table.parts[i].bootLabel);
		fprintf(f, "\ttable=/dev/%.3s\n", table.parts[i].device);

		if (strncmp(table.parts[i].device, hdname, 3)) {
		    /* boot off the second drive, so reverse the BIOS maps */
		    fprintf(f, "\tmap-drive=0x80\n");
		    fprintf(f, "\t   to = 0x81\n");
		    fprintf(f, "\tmap-drive=0x81\n");
		    fprintf(f, "\t   to = 0x80\n");
		}
	    }
	}
    }

    fclose(f);
   return 0;
}
static int mkinitrd(char * kernelVersion, char * initrdImage, int issmp) {
    char * argv[] = { "/sbin/mkinitrd", "-f", initrdImage, "--ifneeded",
		      kernelVersion, NULL };
    int rc = 0;
    static int alreadyHappened[2] = { 0, 0 };
    char ver[60];

    if (alreadyHappened[issmp]) return 0;

    if (issmp)
	sprintf(ver, "%ssmp", kernelVersion);
    else
	sprintf(ver, "%s", kernelVersion);
    argv[4] = ver;

    if (loadModule("loop", DRIVER_OTHER, DRIVER_MINOR_NONE, NULL))
	return INST_ERROR;

    winStatus(32, 3,
#ifdef __sparc__
//	      _("SILO"),
#error "unsupported architecture";
#else
	      _("GRUB"),
#endif
	      _("Creating initial ramdisk..."));
/*
    umount("/mnt/proc");
    rc = doMount("/proc", "/mnt/proc", "proc", 0, 0);
    if (rc) {
    	newtPopWindow();
	return rc;
    }
*/
    rc = runProgramRoot(RUN_LOG, "/mnt", "/sbin/mkinitrd", argv);
    newtPopWindow();

/*    umount("/mnt/proc");*/

    removeModule("loop");

    if (rc) {
	unlink("/mnt/boot/initrd");
    } else {
	alreadyHappened[issmp] = 1;
    }

    return rc;
}

static int noSpaceFilter(newtComponent entry, void * data, int ch, int cursor) {
    if (ch == ' ')
	return 0;
    return ch;
}

int systemRunProgramRoot(enum runType runType, char * root, char * name, char ** args);
int systemRunProgramIORoot(enum runType runType, char * root, char * name,
				char ** args, char * in, char ** out);

int systemRunProgramRoot(enum runType runType, char * root, char * name,
		   char ** args) {
    return systemRunProgramIORoot(runType, root, name, args, NULL, NULL);
}
int systemRunProgramIORoot(enum runType runType, char * root, char * name,
		char ** args, char * in, char ** out) {

    char * buf;
    char *szExecPath;
    int infd, outfd, errfd, status, i;
    int resultfd = 0;
    pid_t pid;
    char ** currarg;
    int inputPipe[2], outputPipe[2];
    char resultbuf[200];
    char * result;
    int resultSize;
    char fullname[200];

    /* if "in" is set, it better be short. We assume one write() call is
       enough */
    if (root) {
	sprintf(fullname, "%s/%s", root, name);
    } else
	strcpy(fullname, name);

    i = 0;
    i = strlen(name) + 50;

    currarg = args;
    while (*currarg) {
	i += strlen(*currarg) + 1;
	currarg++;
    }

    szExecPath = alloca( i );
    szExecPath[ 0 ] = 0;

    currarg = args;

    while ( *currarg ) {
	strcat( szExecPath, *currarg );
	currarg++;
    }

    buf = alloca(i);

    if (testing)
	strcpy(buf, "if I weren't testing I would run:\n\n");
    else
	strcpy(buf, "running: ");

    strcat(buf, name);
    strcat(buf, " ");

    currarg = args;
    while (*currarg) {
	strcat(buf, *currarg);
	strcat(buf, " ");
	currarg++;
    }

    if (testing) {
	newtComponent t, f, succeed, fail;

	newtCenteredWindow(45, 15, "Running");

	succeed = newtButton(8, 10, "Succeed");
	fail = newtButton(28, 10, "Fail");
	t = newtTextbox(2, 1, 40, 13, NEWT_TEXTBOX_WRAP | NEWT_TEXTBOX_SCROLL);
	newtTextboxSetText(t, buf);
	f = newtForm(NULL, NULL, 0);

	newtFormAddComponents(f, t, succeed, fail, NULL);

	t = newtRunForm(f);

	newtFormDestroy(f);
	newtPopWindow();

	return t == fail;
    }

    if (access(fullname,  X_OK))  {
	logMessage("cannot run %s: %s", fullname, strerror(errno));
	newtWinMessage("Error", "Ok", "I cannot run %s: %s", fullname,
			    strerror(errno));//
	return -1;
    }


    logMessage(buf);

    if (root)
	logMessage("    root is %s", root);

    if (in) {
	pipe(inputPipe);
	write(inputPipe[1], in, strlen(in));
	close(inputPipe[1]);
	infd = inputPipe[0];
    } else {
	infd = open("/dev/null", O_RDONLY);
    }

    errfd = open("/dev/tty5", O_APPEND | O_CREAT);
    if (errfd < 0)
	errfd = open("/tmp/exec.log", O_APPEND | O_CREAT);

    if (out) {
	pipe(outputPipe);
	outfd = outputPipe[1];
	resultfd = outputPipe[0];
    } else if (runType & RUN_LOG) {
	outfd = open("/dev/tty5", O_RDWR);
	if (outfd < 0)
	    outfd = open("/tmp/exec.log", O_APPEND | O_CREAT);
    } else
	outfd = open("/dev/null", O_RDWR);

    if (!(pid = fork())) {
	close(0);
	close(1);
	close(2);
	
	if (root) {
	    chroot(root);
	    chdir("/");
	}

	dup2(infd, 0);
	dup2(outfd, 1);
	dup2(errfd, 2);

	close(infd);
	close(outfd);
	close(errfd);
	if (out) close(resultfd);

	//execv(name, args);

	system( szExecPath ); //buf
	logMessage("exec of %s", szExecPath );/**/
	logMessage("exec of %s failed: %s", name, strerror(errno));
	exit(-1);
    }

    close(infd);
    close(outfd);
    close(errfd);

    if (out) {
	resultSize = 0;
	result = NULL;

	do {
	    i = read(resultfd, resultbuf, sizeof(resultbuf));

	    if (!result) {
		result = malloc(i + 1);
	    } else
		result = realloc(result, resultSize + i + 1);
	    memcpy(result + resultSize, resultbuf, i);
	    resultSize += i;
	    result[resultSize] = '\0';
	} while (i > 0);

        close(resultfd);
	*out = result;
    }

    waitpid(pid, &status, 0);

    if (WIFEXITED(status))
	return WEXITSTATUS(status);

    return -1;
}


static char *ConvLinuxDev2GRUB( char *a_szConvDevName, char *a_szDeviceName) {
char *cpPointer, *cpDeviceNamePtr;
char szTmpDriveNum[ 2 ];
char szTmpPartitionNum[ 255 ];	
int iMaxDevNum, i, numDrives;
char ** drives;
char cDriveStart;
	cDriveStart = 0;
	iMaxDevNum = 0;
	szTmpDriveNum[ 0 ] = 0; szTmpDriveNum[ 1 ] = 0;
	/*!!!!!!!!*/
	 if ( getDriveList(&drives, &numDrives ) ) return NULL;
	/*!!!!!!!*/
	cpPointer = strstr( a_szDeviceName, "/dev/" );
	if ( cpPointer ) cpDeviceNamePtr = cpPointer + 5;
	else cpDeviceNamePtr = a_szDeviceName;
	cpPointer = strstr( cpDeviceNamePtr, "fd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strncpy( a_szConvDevName, cpPointer, 3 ); //Copy also one digit number like fd0
		a_szConvDevName[ 3 ] = 0;		
	}
	cpPointer = strstr( cpDeviceNamePtr, "sd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strcpy( a_szConvDevName, "hd" );
		//FIXME: SCSI disks in BIOSes is last disks
                for (i = 0; drives[i]; i++) {
			if ( drives[i][ 0 ] == 'h' ) {
				if( drives[i][ 2 ] > cDriveStart ) {
					cDriveStart = drives[i][ 2 ];
					iMaxDevNum ++;
				}
			}
		}
		//****************************************

		for (i = 0; drives[i]; i++) {
			if( *( cpPointer + 2 ) > drives[i][ 2 ]  )  iMaxDevNum ++;
		}
		szTmpDriveNum[ 0 ] = iMaxDevNum + 0x30;

		strcat( a_szConvDevName, szTmpDriveNum ); //Copy drive number
		if ( *( cpPointer + 3 ) ) {
			sprintf( szTmpPartitionNum, ",%d", atoi( cpPointer + 3 ) - 1 );
			strcat( a_szConvDevName, szTmpPartitionNum );//copy partition number					
		}
	}
	cpPointer = strstr( cpDeviceNamePtr, "hd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strcpy( a_szConvDevName, "hd" );
/*
		if ( *( cpPointer + 2 ) >= 'a' && *( cpPointer + 2 ) <= 'z' )
			szTmpDriveNum[ 0 ] = ( *( cpPointer + 2 ) - 'a' ) + 0x30;
*/
		for (i = 0; drives[i]; i++) {
			if( *( cpPointer + 2 ) > drives[i][ 2 ]  )  iMaxDevNum ++;
		}

		szTmpDriveNum[ 0 ] = iMaxDevNum + 0x30;
		strcat( a_szConvDevName, szTmpDriveNum ); //Copy drive number
		
		if ( *( cpPointer + 3 ) ) {
			sprintf( szTmpPartitionNum, ",%d", atoi( cpPointer + 3 ) - 1 );
			strcat( a_szConvDevName, szTmpPartitionNum );//copy partition number					
		}
	}
	return a_szConvDevName;
}
#define SKIP_GRUB 1000

static int GRUBWhere(char * hdName, char * bootDevice, char ** where) {
    char * format = "/dev/%-11s    %s";
    char ** items;
    int which = 0, rc;

    items = alloca(sizeof(*items) * 3);
    items[0] = alloca(80);
    items[1] = alloca(80);
    items[2] = NULL;

    sprintf(items[0], format, hdName, _("Master Boot Record"));
    sprintf(items[1], format, bootDevice, _("First sector of boot partition"));

#ifdef __i386__
    rc = newtWinMenu(_("GRUB Installation"), _("Where do you want to install "
		       "the bootloader?"), 50, 5, 5, 4, items, &which,
		     _("Ok"), _("Skip"), _("Back"), NULL);
#elif defined(__sparc__)
/*
    rc = newtWinMenu(_("SILO Installation"), _("Where do you want to install "
		       "the bootloader?"), 50, 5, 5, 4, items, &which,
		     _("Ok"), _("Skip"), _("Back"), NULL);
*/
#error "unsupported architecture";
#endif		

    if (rc == 3) return INST_CANCEL;
    if (rc == 2) return SKIP_GRUB;

    switch (which) {
      case 0: 		*where = hdName; break;
      case 1: 		*where = bootDevice; break;
    }

    return 0;
}

static void editBootLabel(struct partition * item) {
    newtComponent form, entry, okay, cancel, clear, answer;
    char buf[50];
    char * entryValue;
    newtGrid subgrid, grid, buttons;

    form = newtForm(NULL, NULL, 0);

    strcpy(buf, "/dev/");
    strcat(buf, item->device);

    entry = newtEntry(-1, -1, item->bootLabel, 20, &entryValue,
		      NEWT_ENTRY_SCROLL | NEWT_ENTRY_RETURNEXIT);
    newtEntrySetFilter(entry, noSpaceFilter, NULL);


    subgrid = newtCreateGrid(2, 2);
    newtGridSetField(subgrid, 0, 0, NEWT_GRID_COMPONENT,
		     newtLabel(-1, -1, _("Device:")),
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(subgrid, 1, 0, NEWT_GRID_COMPONENT,
		     newtLabel(-1, -1, buf),
		     1, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(subgrid, 0, 1, NEWT_GRID_COMPONENT,
		     newtLabel(-1, -1, _("Boot label:")),
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(subgrid, 1, 1, NEWT_GRID_COMPONENT, entry,
		     1, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);

    buttons = newtButtonBar(_("Ok"), &okay, _("Clear"), &clear,
			    _("Cancel"), &cancel, NULL);

    grid = newtCreateGrid(1, 2);
    newtGridSetField(grid, 0, 0, NEWT_GRID_SUBGRID, subgrid,
		     0, 0, 0, 0, 0, 0);
    newtGridSetField(grid, 0, 1, NEWT_GRID_SUBGRID, buttons,
		     0, 1, 0, 0, 0, NEWT_GRID_FLAG_GROWX);

    newtGridAddComponentsToForm(grid, form, 1);
    newtGridWrappedWindow(grid, _("Edit Boot Label"));
    newtGridFree(grid, 1);

    do {
	answer = newtRunForm(form);

	if (answer == clear)
	    newtEntrySet(entry, "", 1);
    } while (answer == clear);

    if (answer != cancel) {
	if (item->bootLabel) free(item->bootLabel);

	if (strlen(entryValue))
	    item->bootLabel = strdup(entryValue);
	else
	    item->bootLabel = NULL;
    }

    newtPopWindow();
}	


static int doinstallGRUB(char * prefix, char * dev, char * rootdev,
			 char * bootdev,
			 struct partitionTable table,
			 char * append, char * kernelVersion,
			 char * hdname, int XInUse ){
    char filename[100], szTransFileName[ 100 ];
    FILE * f;
//    char * argv[] = { GRUB_BINARY, NULL, NULL };
    char **argv; //6-255
    int i;
    int rc;
    struct stat sb;
    int useInitrd = 0;
    char relinitrdImage[50], absinitrdImage[55];
    char buff[1024];
//    int pass;
    static int firstTime = 1;
    static int issmp = 0;
    char relsmpinitrdImage[50];
    int usesmpInitrd = 0;
    char szConvDevName[ 255 ];
    char szInstallDevName[ 10 ];
    int iTmp, iCount;
    int iDefBootNum = -1;
    char *szTransLang;
    char szInstPrefix[ 20 ];
    char szKernelPrefix[ 20 ];

    if ( !strcmp( rootdev, bootdev ) )
	strcpy( szKernelPrefix, "/boot/" );
    else
	strcpy( szKernelPrefix, "/" );

    argv = alloca(sizeof(*argv) * 4);
    for ( iTmp = 0; iTmp < 4; iTmp++ ) argv[ iTmp ] = alloca(1000);


    sprintf(relinitrdImage, "/boot/initrd-%s.img", kernelVersion);
    strcpy(absinitrdImage, "/mnt");
    strcat(absinitrdImage, relinitrdImage);

    if (testing) return 0;

    if ((issmp = detectSMP()) == -1) {
	logMessage("Error trying to detect SMP machine");
	issmp = 0;
    }
//Must be placed not here but why not here?
//It try to relink for kernel files in case of SMP support or different kernel version
//Now GRUB must use only vmlinuz link
    if ( link_boot_files( issmp, "/mnt/boot/", kernelVersion ) )
	logMessage("Can`t modificate link files");
//!!!!!!!!!!!

    if (issmp) {

	sprintf(buff, "/mnt/boot/%s%ssmp", KERNEL_SMPIMAGE, kernelVersion);
	if (stat(buff, &sb)) {
	    logMessage("SMP machine, but no SMP kernel found");
	    issmp = 0;
	}
    }
    if (issmp) {
	char abssmpinitrdImage[55];
	sprintf(relsmpinitrdImage, "/boot/initrd-%ssmp.img", kernelVersion);
	if (mkinitrd(kernelVersion, relsmpinitrdImage, 1)){
	    issmp = 0;	
	}
	else {
	    strcpy(abssmpinitrdImage, "/mnt");
	    strcat(abssmpinitrdImage, relsmpinitrdImage);
	
	    if (!stat (abssmpinitrdImage, &sb))
		usesmpInitrd = 1;
	}
    } else {
    	if (mkinitrd(kernelVersion, relinitrdImage, 0))
		return INST_ERROR;
	if (!stat(absinitrdImage, &sb))
		useInitrd = 1;

    }	
     //Backup LILO Configuratuion for Emergency usages
    GoodOldLILOConfWrite( prefix, dev, rootdev, bootdev, table, append, kernelVersion, hdname,
			    issmp, usesmpInitrd, relsmpinitrdImage, useInitrd,  relinitrdImage  );
    //


#ifdef __sparc__
/*	sprintf(filename, "%s/silo.conf", prefix);*/
#error "unsupported architecture";
#else /* !__sparc__ */
//	sprintf(filename, "%s/GRUB.conf", prefix);
	sprintf(filename, "%s/grub.conf", prefix);
#endif /* __sparc__ */

    /* why not? */
    if (firstTime) {
   	rename("/mnt/etc/grub.conf", "/mnt/etc/grub.conf.rpmsave");
	firstTime = 0;
    }

    f = fopen(filename, "w");
    if (!f) {
	errorWindow("cannot create GRUB config file: %s");
	return INST_ERROR;
    }

    logMessage("writing GRUB config to %s", filename);

    fprintf( f, _("# This file has been generated by Best Linux install\n\n") );
    fprintf( f, _("# Options\n") );
/*HARD DEBUG*/
//    fprintf( f, "# HARD DEBUG=prefix=%s, dev=%s, rootdev=%s, bootdev=%s\n", prefix, dev, rootdev, bootdev );
/*REMOVE AFTER TEST*/
    fprintf( f, "color=light-gray/blue black/light-gray\n" );
    //Here get current language and setup specifical language file
    szTransLang = getenv("LANG");
    if ( szTransLang && strcmp( szTransLang, "en" ) ) {
    	sprintf( szTransFileName,"/mnt/boot/grub/%s.grub", szTransLang );
	if ( !access( szTransFileName, F_OK ) )
		fprintf( f, "messages=/boot/grub/%s.grub\n", szTransLang );
    }

    for ( iCount = 0; iCount < table.count; iCount++ ) {
	if ( table.parts[ iCount ].defaultBoot ) {
		iDefBootNum = iCount;
		break;
	}
    }

    if ( iDefBootNum == -1 ) iDefBootNum = 0;
    else {
	if ( XInUse ) iDefBootNum += 3;
	else iDefBootNum += 2;
   }

    fprintf( f, "default=%d\n", iDefBootNum );
    fprintf( f, "timeout=30\n\n" );

    for (i = 0; i < table.count; i++) {
    	if (table.parts[i].type == PART_EXT2) {
		//TODO: Strange but if we found another Linux PART_EXT2 with different version of Linux we do also that`s strings.
		// But let`s go edit it. :(
		if ( !strcmp( table.parts[i].device, rootdev ) || //Added not 100% checked
		     !strcmp( table.parts[i].device, bootdev ) ) {
			//Added
                	if ( XInUse ) {
				//Run level 5
				fprintf( f, "\ntitle=%s\n", _("Best Linux graphical mode") );	
				fprintf( f, "root=(%s)\n", ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device ) );
				fprintf(f, "kernel=%svmlinuz 5 ", szKernelPrefix );
       				if ( append ) fprintf( f, " %s ", append );
				fprintf(f, "root=/dev/%s\n", rootdev );
				if (useInitrd)
					fprintf(f, "initrd=%s\n", relinitrdImage);
				if ( usesmpInitrd )
					fprintf(f, "initrd=%s\n", relsmpinitrdImage );
					

                	}
                	//Run level 3
			fprintf( f, "\ntitle=%s\n", _ ("Best Linux text mode") );
			fprintf( f, "root=(%s)\n", ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device ) );
			fprintf(f, "kernel=%svmlinuz 3 ", szKernelPrefix );
	         	if ( append ) fprintf( f, " %s ", append );
 			fprintf(f, "root=/dev/%s\n", rootdev );
			if (useInitrd)
				fprintf(f, "initrd=%s\n", relinitrdImage);
			if ( usesmpInitrd )
				fprintf(f, "initrd=%s\n", relsmpinitrdImage );
       			//Run level 1 or failsafe
			fprintf( f, "\ntitle=%s\n", _("Best Linux failsafe mode") );
			fprintf( f, "root=(%s)\n", ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device ) );
			fprintf(f, "kernel=%svmlinuz 1 ", szKernelPrefix );
	         		if ( append ) fprintf( f, " %s ", append );
	 		fprintf(f, "root=/dev/%s\n", rootdev );
			if (useInitrd)
				fprintf(f, "initrd=%s\n", relinitrdImage);
			if ( usesmpInitrd )
				fprintf(f, "initrd=%s\n", relsmpinitrdImage );
			//End add new
			break;
		}
	}
    }
    for (i = 0; i < table.count; i++) {
	if (!table.parts[i].bootLabel) continue;
	if (table.parts[i].type == PART_EXT2) {
		if ( !strcmp( table.parts[i].device, rootdev ) || !strcmp( table.parts[i].device, bootdev ) )
			continue;
		fprintf(f, "\ntitle=%s\n", table.parts[i].bootLabel);
	        	fprintf( f, "root=(%s)\n", ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device ) );
		fprintf(f, "kernel=" KERNEL_PURE_IMAGE " " );
      	        	fprintf(f, "root=/dev/%s\n", table.parts[i].device);
#ifdef __i386__
	    } else {
               if ( table.parts[i].type == PART_DOS || table.parts[i].type == PART_FAT32 ) {
			fprintf(f, "\n\ntitle=%s\n", table.parts[i].bootLabel);
			fprintf( f, "root=(%s)\n", ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device ) );
			fprintf( f, "makeactive\n" );
			fprintf( f, "chainloader +1\n" );

			if (strncmp(table.parts[i].device, hdname, 3)) {
			    /* boot off the second drive, so reverse the BIOS maps */

				ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device );
				strncpy( szInstallDevName, szConvDevName, 3 ); //temporary used
				szInstallDevName[ 3 ] = 0; //temporary used

			    fprintf( f, "map (hd0) (%s)\n", szInstallDevName );
	   		    fprintf( f, "map (%s) (hd0)\n", szInstallDevName );
			}
	        } else {
			fprintf(f, "\n\ntitle=%s\n", table.parts[i].bootLabel);
			fprintf( f, "root=(%s)\n", ConvLinuxDev2GRUB( szConvDevName, table.parts[i].device ) );
		
		}	
#endif
	    }
    }

    fclose(f);

    winStatus(35, 3, _("Running"), _("Installing boot loader..."));

#ifdef __i386__
/*
	ConvLinuxDev2GRUB( szConvDevName, dev );
	strcpy( szInstallDevName, szConvDevName );
	if ( !strstr( szInstallDevName, "," ) )
		strcat( szInstallDevName, ",0" );
*/
 	strcpy( argv[ 0 ], "/bin/sh" );
	strcpy ( argv[ 1 ],  "-c" );
	
	/**///printf( "%s - %s - %s\n", rootdev, dev, bootdev );
/*	sprintf( argv[ 2 ], "/sbin/grub --batch  <<EOT\nroot=(%s)\ninstall=/boot/grub/stage1 (%s) /boot/grub/stage2 0x8000 p /etc/grub.conf\nquit\nEOT\n",
				 ConvLinuxDev2GRUB( szConvDevName, rootdev ),
				ConvLinuxDev2GRUB( szInstallDevName, dev ) );
*/
	if ( !strcmp( rootdev, bootdev ) )
		sprintf( szInstPrefix, "(%s)/boot/grub/", ConvLinuxDev2GRUB( szConvDevName, rootdev ) );
	else
		sprintf( szInstPrefix, "(%s)/grub/", ConvLinuxDev2GRUB( szConvDevName, bootdev ) );
	sprintf( argv[ 2 ], "/sbin/grub --no-curses --batch  <<EOT\nroot=(%s)\ninstall=%sstage1 d (%s)  %sstage2 0x8000 p (%s)/etc/grub.conf\nquit\nEOT\n",
				ConvLinuxDev2GRUB( szConvDevName, rootdev ), szInstPrefix,
				ConvLinuxDev2GRUB( szInstallDevName, dev ),
				szInstPrefix, ConvLinuxDev2GRUB( szConvDevName, rootdev ) );



        argv[ 3 ] = NULL;

//      system( "umount /proc" );
//      system( "mount -t proc proc /mnt/root/proc" );
        umount("/mnt/proc");
        rc = doMount("/proc", "/mnt/proc", "proc", 0, 0);
        if (rc) {
         	newtPopWindow();
	return rc;
        }
        rc = runProgramRoot(RUN_LOG, "/mnt", "/bin/sh", argv);
        umount("/mnt/proc");

//      system( "umount /mnt/root/proc" );
//      system( "mount -t proc proc /proc" );
	
/*
#elif __sparc__

        if (strcmp(hdname, dev))
            argv[1] = "-t";
	umount("/mnt/proc");
	rc = doMount("/proc", "/mnt/proc", "proc", 0, 0);
	if (rc) {
	    newtPopWindow();
	    return rc;
	}
	rc = runProgramRoot(RUN_LOG, "/mnt", "sbin/silo", argv);
	umount("/mnt/proc");
*/
#else

#error unsupported architectures
#endif
    newtPopWindow();

    if (rc)
	return INST_ERROR;

    return 0;
}

static void formatEntry(char * buf, struct partition * part) {
    sprintf(buf, "/dev/%-5s  %-25s %-7s %-10s", part->device,
	    part->tagName,
	    part->defaultBoot ? "   *" : "",
	    part->bootLabel ? part->bootLabel : "");
}

static int getBootLabels(struct partitionTable table, struct fstab fstab, char *where ) {
    newtComponent f, okay, text, listbox, label, cancel, edit;
    struct newtExitStruct answer;
    char buf[80];
    int i, j;
    int foundDos = 0;
    int mustAsk = 0;
    int * map;
    struct partition * curr;
    int * currNum;
    int count;
    int done;
    int defaultBootPart = 0;
    char * reflowedText;
    int width, height;
    newtGrid buttons, grid;

    reflowedText = newtReflowText(
		       _("The boot manager Best Linux uses can boot other "
                         "operating systems as well. You need to tell me "
                         "what partitions you would like to be able to boot "
                         "and what label you want to use for each of them."),
			 60, -5, -5, &width, &height);
    text = newtTextbox(-1, -1, width, height, NEWT_TEXTBOX_WRAP);
    newtTextboxSetText(text, reflowedText);
    free(reflowedText);

    f = newtForm(NULL, NULL, 0);

    sprintf(buf, "%-10s  %-25s %-7s %-10s", _("Device"), _("Partition type"),
			_("Default"), _("Boot label"));
    label = newtLabel(-1, -1, buf);

    listbox = newtListbox(-1, -1, 10 - height,
			  NEWT_LISTBOX_RETURNEXIT | NEWT_FLAG_SCROLL);
    map = alloca(sizeof(int) * table.count);

    for (i = 0, count = 0; i < table.count; i++) {
	if (table.parts[i].type != PART_SWAP &&
	    table.parts[i].type != PART_IGNORE &&
#ifdef __sparc__
/*	      table.parts[i].type != PART_OTHER &&*/
#error "unsupported architecture";
#endif
	    (table.parts[i].type != PART_FAT32 || !foundDos) &&
	    (table.parts[i].type != PART_DOS || !foundDos)) {

	    if (table.parts[i].type == PART_DOS ||
	        table.parts[i].type == PART_FAT32) {
		foundDos = 1;
	    }

	    if (table.parts[i].type == PART_EXT2) {
		for (j = 0; j < fstab.numEntries; j++) {
		    if (!strcmp(table.parts[i].device, fstab.entries[j].device))
			break;
		}

		if (j < fstab.numEntries && !table.parts[i].bootLabel)
		    continue;
		//TODO: Add here ignore of current installation path
		if ( !strcmp( table.parts[i].device, where ) ) continue;
	    }

	    if (!table.parts[i].bootLabel ||
		strcmp(table.parts[i].bootLabel, "linux")) mustAsk = 1;

	    if (table.parts[i].defaultBoot)
		defaultBootPart = count;

	    map[count] = i;
	    formatEntry(buf, table.parts + i);
	    newtListboxAddEntry(listbox, buf, map + count++);
	}
    }

    /* we add stuff to the form here so we can actually destroy it if
       we don't need this window after all */
    newtFormAddComponents(f, text, label, listbox, NULL);

    if (!mustAsk) {
	newtFormDestroy(f);
	return 0;
    }

    newtPushHelpLine("<F2> Selects the default partition");

    buttons = newtButtonBar(_("Ok"), &okay, _("Edit"), &edit,
			    _("Back"), &cancel, NULL);

    newtFormAddComponents(f, okay, edit, cancel, NULL);

    newtFormAddHotKey(f, NEWT_KEY_F2);

    grid = newtCreateGrid(1, 4);

    newtGridSetField(grid, 0, 0, NEWT_GRID_COMPONENT, text,
		     0, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(grid, 0, 1, NEWT_GRID_COMPONENT, label,
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(grid, 0, 2, NEWT_GRID_COMPONENT, listbox,
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(grid, 0, 3,  NEWT_GRID_SUBGRID, buttons,
		     0, 1, 0, 0, 0, NEWT_GRID_FLAG_GROWX);

    newtGridWrappedWindow(grid, _("Bootable Partitions"));
    newtGridFree(grid, 1);

    done = 0;
    while (!done) {
	newtFormRun(f, &answer);

	if (answer.reason == NEWT_EXIT_HOTKEY) {
	    if (answer.u.key == NEWT_KEY_F12) {
		done = 1;
	    } else if (answer.u.key == NEWT_KEY_F2) {
		currNum = newtListboxGetCurrent(listbox);
		curr = table.parts + *currNum;

		if (!curr->bootLabel) {
		    newtWinMessage("Boot Partition", "Ok", "You cannot mark a "
				  "partition as the default partition to "
				  "boot from unless that partition has "
				  "been assigned a boot label.");
		} else{
		    for (i = 0; i < count; i++) {
			if (table.parts[map[i]].defaultBoot) {
			    table.parts[map[i]].defaultBoot = 0;
			    formatEntry(buf, table.parts + map[i]);
			    newtListboxSetEntry(listbox, i, buf);
			    break;
			}
		    }

		    curr->defaultBoot = 1;
		    formatEntry(buf, curr);
		    newtListboxSetEntry(listbox, currNum - map, buf);
		}
	    }
	} else {
	    if (answer.u.co == edit || answer.u.co== listbox) {
		currNum = newtListboxGetCurrent(listbox);
		curr = table.parts + *currNum;
		editBootLabel(curr);

		if (!curr->bootLabel && curr->defaultBoot) {
		    curr->defaultBoot = 0;
		    if (table.parts[map[defaultBootPart]].bootLabel) {
			table.parts[map[defaultBootPart]].defaultBoot = 1;

			formatEntry(buf, table.parts + map[defaultBootPart]);
			newtListboxSetEntry(listbox, defaultBootPart, buf);
		    }
		}

		formatEntry(buf, curr);
		newtListboxSetEntry(listbox, currNum - map, buf);
	    } else
		done = 1;
	}
    }

    newtPopHelpLine();

    newtFormDestroy(f);
    newtPopWindow();

    if (answer.reason == NEWT_EXIT_COMPONENT && answer.u.co == cancel)
	return INST_CANCEL;
    else
	return 0;
}

static int getAppendLine(char ** line/*, int * linear*/) {
    newtComponent form, text, entry, okay, cancel, answer;
//    newtComponent linearCheck;
    char * result = NULL;
//    char linearChar = (*linear) ? '*' : ' ';
    newtGrid grid, buttons, subgrid;

    text = newtTextboxReflowed(0, 0,
		     _("A few systems will need to pass special options "
		       "to the kernel at boot time for the system to function "
		       "properly. If you need to pass boot options to the "
		       "kernel, enter them now. If you don't need any or "
		       "aren't sure, leave this blank."), 53, 10, 0, 0);

    form = newtForm(NULL, NULL, 0);

    entry = newtEntry(-1, -1, *line, 48, &result,
			NEWT_ENTRY_SCROLL | NEWT_ENTRY_RETURNEXIT);

#ifndef __sparc__
  /*
	linearCheck = newtCheckbox(-1, -1,
			     _("Use linear mode (needed for some SCSI drives)"),
			     linearChar, NULL, &linearChar);
*/
#endif

    buttons = newtButtonBar(_("Ok"), &okay, _("Back"), &cancel, NULL);
    subgrid = newtGridVStacked(NEWT_GRID_COMPONENT, entry,
#ifndef __sparc__
//				  NEWT_GRID_COMPONENT, linearCheck,
#endif
			       NULL);
    grid = newtGridBasicWindow(text, subgrid, buttons);

    newtGridAddComponentsToForm(grid, form, 1);
    newtFormSetCurrent(form, okay);

#ifdef __sparc__
/*	newtGridWrappedWindow(grid, _("Silo Installation"));*/
#error "unsupported architecture";
#else
	newtGridWrappedWindow(grid, _("GRUB Installation"));
#endif

    newtGridFree(grid, 1);

    answer = newtRunForm(form);

    newtPopWindow();

    if (answer == cancel) {
	newtFormDestroy(form);
	return INST_CANCEL;
    }

//    *linear = linearChar != ' ';

    if (!strlen(result))
	*line = NULL;
    else
	*line = strdup(result);

    newtFormDestroy(form);

    return 0;
}

#define GRUB_WHERE 2
#define GRUB_LABELS 3
#define GRUB_INSTALL 4
#define GRUB_APPEND 5
#define GRUB_DONE 20

int installGRUB(char * prefix, struct partitionTable table, struct fstab fstab,
		char * kernelVersion, int flags, char * options,  int XInUse ) {
    char * rootDevice, * bootDevice = NULL;
    char * hdName;
    char * where = NULL;
    char * append = NULL;
    char * chptr = NULL;
    int i;
    int rc;
    int stage = GRUB_WHERE;

    int foundDos = 0;

    hdName = default_boot_hd;

    for (i = 0; i < fstab.numEntries; i++) {
	if (!strcmp(fstab.entries[i].mntpoint, "/boot")) break;
    }

    if (i < fstab.numEntries)
	bootDevice = fstab.entries[i].device;

    for (i = 0; i < fstab.numEntries; i++) {
	if (!strcmp(fstab.entries[i].mntpoint, "/")) break;
    }

    rootDevice = fstab.entries[i].device;
    if (!bootDevice) {
	bootDevice = rootDevice;
    }

    for (i = 0; i < table.count; i++) {
	if (!strcmp(table.parts[i].device, bootDevice)) {
	    table.parts[i].bootLabel = strdup("linux");
//	    table.parts[i].defaultBoot = 1;
	} else if ((table.parts[i].type == PART_DOS ||
	    table.parts[i].type == PART_FAT32) && !foundDos) {
	    table.parts[i].bootLabel = strdup("dos");
	    foundDos = 1;
	}
    }

    if (flags & LILO_ON_MBR)
	where = hdName;
    else if (flags & LILO_ON_PARTITION)
	where = bootDevice;

    if (flags & LILO_USE_OPTIONS)
	append = options;

    if (where) {

	rc = doinstallGRUB(prefix, where, rootDevice, bootDevice, table,
			    append, kernelVersion, hdName,  XInUse);
	if (rc == INST_ERROR) return INST_ERROR;
	stage = GRUB_DONE;
    }

    while (stage != GRUB_DONE) {
	switch (stage) {
	  case GRUB_WHERE:
	    rc = GRUBWhere(hdName, bootDevice, &where);
            if (rc == SKIP_GRUB ) return 0;
	    if (rc) return rc;
	    stage = GRUB_APPEND;
	    break;

	  case GRUB_APPEND:
	    chptr = append;
	    rc = getAppendLine(&chptr/*, &linear*/);

	    if (rc == INST_ERROR) return INST_ERROR;
	    if (rc == INST_CANCEL)
		stage = GRUB_WHERE;
	    else {
		stage = GRUB_LABELS;

		if (append) free(append);
		if (chptr) {
		    append = alloca(strlen(chptr) + 1);
		    strcpy(append, chptr);
		    free(chptr);
		} else {
		    append = NULL;
		}
	    }

	    break;

	  case GRUB_LABELS:
	    rc = getBootLabels(table, fstab, rootDevice);
	    if (rc == INST_ERROR) return INST_ERROR;
	    if (rc == INST_CANCEL)
		stage = GRUB_APPEND;
	    else
		stage = GRUB_INSTALL;
	    break;

	  case GRUB_INSTALL:
	    rc = doinstallGRUB(prefix, where, rootDevice, bootDevice,
				table, append, kernelVersion, hdName,  XInUse);
	    if (rc == INST_ERROR) return INST_ERROR;
	    stage = GRUB_DONE;
	    break;
	}
    }

    return 0;
}
int makeBootdiskWithGRUB(char * prefix, char * kernelVersion) {
    char * argv[] = { "/sbin/mkbootdisk", "--noprompt", "-g", "--device", "/dev/fd0", kernelVersion, NULL };
    int rc;

#ifdef __sparc__
    if (loadModule("romfs", DRIVER_OTHER, DRIVER_MINOR_NONE, NULL))
        return INST_ERROR;
    /* Find out if the only floppy in the system is /dev/fd1 */
    rc = open("/mnt/dev/fd0", O_RDONLY);
    if (rc < 0 && errno == ENXIO)
        argv[3] = "/dev/fd1";
    else if (rc >= 0)
    	close(rc);
#endif
    if (loadModule("loop", DRIVER_OTHER, DRIVER_MINOR_NONE, NULL))
	return INST_ERROR;
	
    winStatus(32, 3, _("Bootdisk"), _("Creating bootdisk..."));

//   system( "umount /proc" );
//   system( "mount -t proc proc /mnt/root/proc" );
    umount("/mnt/proc");
    rc = doMount("/proc", "/mnt/proc", "proc", 0, 0);
    if (rc) {
    	newtPopWindow();
	return rc;
    }
    rc = runProgramRoot(RUN_LOG, "/mnt", "/sbin/mkbootdisk", argv);
    newtPopWindow();

    umount("/mnt/proc");

//   system( "umount /mnt/root/proc" );
//   system( "mount -t proc proc /proc" );

    removeModule("loop");

    return rc;
}


